@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016-2022 NVIDIA Corporation
+    Copyright (c) 2016-2023 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -22,6 +22,7 @@
 *******************************************************************************/
 
 #include "uvm_mem.h"
+#include "uvm_hal_types.h"
 #include "uvm_mmu.h"
 #include "uvm_processors.h"
 #include "uvm_va_space.h"
@@ -67,26 +68,15 @@ static bool vidmem_can_be_mapped(uvm_mem_t *vidmem, bool is_user_space)
     return true;
 }
 
-static bool sysmem_can_be_mapped(uvm_mem_t *sysmem)
-{
-    UVM_ASSERT(uvm_mem_is_sysmem(sysmem));
-
-    // If SEV is enabled, only unprotected memory can be mapped
-    if (g_uvm_global.sev_enabled)
-        return uvm_mem_is_sysmem_dma(sysmem);
-
-    return true;
-}
-
 static bool mem_can_be_mapped_on_cpu(uvm_mem_t *mem, bool is_user_space)
 {
     if (uvm_mem_is_sysmem(mem))
-        return sysmem_can_be_mapped(mem);
+        return true;
 
     if (!vidmem_can_be_mapped(mem, is_user_space))
         return false;
 
-    return mem->backing_gpu->parent->numa_info.enabled && PAGE_ALIGNED(mem->chunk_size);
+    return mem->backing_gpu->mem_info.numa.enabled && PAGE_ALIGNED(mem->chunk_size);
 }
 
 static bool mem_can_be_mapped_on_cpu_kernel(uvm_mem_t *mem)
@@ -99,10 +89,22 @@ static bool mem_can_be_mapped_on_cpu_user(uvm_mem_t *mem)
     return mem_can_be_mapped_on_cpu(mem, true);
 }
 
+static bool sysmem_can_be_mapped_on_gpu(uvm_mem_t *sysmem)
+{
+    UVM_ASSERT(uvm_mem_is_sysmem(sysmem));
+
+    // In Confidential Computing, only unprotected memory can be mapped on the
+    // GPU
+    if (g_uvm_global.conf_computing_enabled)
+        return uvm_mem_is_sysmem_dma(sysmem);
+
+    return true;
+}
+
 static bool mem_can_be_mapped_on_gpu(uvm_mem_t *mem, uvm_gpu_t *gpu, bool is_user_space)
 {
     if (uvm_mem_is_sysmem(mem))
-        return sysmem_can_be_mapped(mem);
+        return sysmem_can_be_mapped_on_gpu(mem);
 
     if (!vidmem_can_be_mapped(mem, is_user_space))
         return false;
@@ -125,12 +127,12 @@ bool uvm_mem_mapped_on_gpu_user(uvm_mem_t *mem, uvm_gpu_t *gpu)
     if (mem->user == NULL)
         return false;
 
-    return uvm_global_processor_mask_test(&mem->user->mapped_on, gpu->global_id);
+    return uvm_processor_mask_test(&mem->user->mapped_on, gpu->id);
 }
 
 bool uvm_mem_mapped_on_gpu_kernel(uvm_mem_t *mem, uvm_gpu_t *gpu)
 {
-    return uvm_global_processor_mask_test(&mem->kernel.mapped_on, gpu->global_id);
+    return uvm_processor_mask_test(&mem->kernel.mapped_on, gpu->id);
 }
 
 bool uvm_mem_mapped_on_cpu_user(uvm_mem_t *mem)
@@ -138,12 +140,12 @@ bool uvm_mem_mapped_on_cpu_user(uvm_mem_t *mem)
     if (mem->user == NULL)
         return false;
 
-    return uvm_global_processor_mask_test(&mem->user->mapped_on, UVM_GLOBAL_ID_CPU);
+    return uvm_processor_mask_test(&mem->user->mapped_on, UVM_ID_CPU);
 }
 
 bool uvm_mem_mapped_on_cpu_kernel(uvm_mem_t *mem)
 {
-    return uvm_global_processor_mask_test(&mem->kernel.mapped_on, UVM_GLOBAL_ID_CPU);
+    return uvm_processor_mask_test(&mem->kernel.mapped_on, UVM_ID_CPU);
 }
 
 static void mem_set_mapped_on_gpu_user(uvm_mem_t *mem, uvm_gpu_t *gpu)
@@ -152,7 +154,7 @@ static void mem_set_mapped_on_gpu_user(uvm_mem_t *mem, uvm_gpu_t *gpu)
     UVM_ASSERT(mem_can_be_mapped_on_gpu_user(mem, gpu));
     UVM_ASSERT(!uvm_mem_mapped_on_gpu_user(mem, gpu));
 
-    uvm_global_processor_mask_set(&mem->user->mapped_on, gpu->global_id);
+    uvm_processor_mask_set(&mem->user->mapped_on, gpu->id);
 }
 
 static void mem_set_mapped_on_gpu_kernel(uvm_mem_t *mem, uvm_gpu_t *gpu)
@@ -160,7 +162,7 @@ static void mem_set_mapped_on_gpu_kernel(uvm_mem_t *mem, uvm_gpu_t *gpu)
     UVM_ASSERT(mem_can_be_mapped_on_gpu_kernel(mem, gpu));
     UVM_ASSERT(!uvm_mem_mapped_on_gpu_kernel(mem, gpu));
 
-    uvm_global_processor_mask_set(&mem->kernel.mapped_on, gpu->global_id);
+    uvm_processor_mask_set(&mem->kernel.mapped_on, gpu->id);
 }
 
 static void mem_set_mapped_on_cpu_user(uvm_mem_t *mem)
@@ -169,7 +171,7 @@ static void mem_set_mapped_on_cpu_user(uvm_mem_t *mem)
     UVM_ASSERT(mem_can_be_mapped_on_cpu_user(mem));
     UVM_ASSERT(!uvm_mem_mapped_on_cpu_user(mem));
 
-    uvm_global_processor_mask_set(&mem->user->mapped_on, UVM_GLOBAL_ID_CPU);
+    uvm_processor_mask_set(&mem->user->mapped_on, UVM_ID_CPU);
 }
 
 static void mem_set_mapped_on_cpu_kernel(uvm_mem_t *mem)
@@ -177,38 +179,38 @@ static void mem_set_mapped_on_cpu_kernel(uvm_mem_t *mem)
     UVM_ASSERT(mem_can_be_mapped_on_cpu_kernel(mem));
     UVM_ASSERT(!uvm_mem_mapped_on_cpu_kernel(mem));
 
-    uvm_global_processor_mask_set(&mem->kernel.mapped_on, UVM_GLOBAL_ID_CPU);
+    uvm_processor_mask_set(&mem->kernel.mapped_on, UVM_ID_CPU);
 }
 
 static void mem_clear_mapped_on_gpu_kernel(uvm_mem_t *mem, uvm_gpu_t *gpu)
 {
-    uvm_global_processor_mask_clear(&mem->kernel.mapped_on, gpu->global_id);
+    uvm_processor_mask_clear(&mem->kernel.mapped_on, gpu->id);
 }
 
 static void mem_clear_mapped_on_gpu_user(uvm_mem_t *mem, uvm_gpu_t *gpu)
 {
     UVM_ASSERT(mem->user != NULL);
 
-    uvm_global_processor_mask_clear(&mem->user->mapped_on, gpu->global_id);
+    uvm_processor_mask_clear(&mem->user->mapped_on, gpu->id);
 }
 
 static void mem_clear_mapped_on_cpu_user(uvm_mem_t *mem)
 {
     UVM_ASSERT(mem->user != NULL);
 
-    uvm_global_processor_mask_clear(&mem->user->mapped_on, UVM_GLOBAL_ID_CPU);
+    uvm_processor_mask_clear(&mem->user->mapped_on, UVM_ID_CPU);
 }
 
 static void mem_clear_mapped_on_cpu_kernel(uvm_mem_t *mem)
 {
-    uvm_global_processor_mask_clear(&mem->kernel.mapped_on, UVM_GLOBAL_ID_CPU);
+    uvm_processor_mask_clear(&mem->kernel.mapped_on, UVM_ID_CPU);
 }
 
 static bool sysmem_mapped_on_gpu_phys(uvm_mem_t *sysmem, uvm_gpu_t *gpu)
 {
     UVM_ASSERT(uvm_mem_is_sysmem(sysmem));
 
-    return uvm_global_processor_mask_test(&sysmem->sysmem.mapped_on_phys, gpu->global_id);
+    return uvm_processor_mask_test(&sysmem->sysmem.mapped_on_phys, gpu->id);
 }
 
 static void sysmem_set_mapped_on_gpu_phys(uvm_mem_t *sysmem, uvm_gpu_t *gpu)
@@ -216,14 +218,14 @@ static void sysmem_set_mapped_on_gpu_phys(uvm_mem_t *sysmem, uvm_gpu_t *gpu)
     UVM_ASSERT(uvm_mem_is_sysmem(sysmem));
     UVM_ASSERT(!sysmem_mapped_on_gpu_phys(sysmem, gpu));
 
-    uvm_global_processor_mask_set(&sysmem->sysmem.mapped_on_phys, gpu->global_id);
+    uvm_processor_mask_set(&sysmem->sysmem.mapped_on_phys, gpu->id);
 }
 
 static void sysmem_clear_mapped_on_gpu_phys(uvm_mem_t *sysmem, uvm_gpu_t *gpu)
 {
     UVM_ASSERT(uvm_mem_is_sysmem(sysmem));
 
-    uvm_global_processor_mask_clear(&sysmem->sysmem.mapped_on_phys, gpu->global_id);
+    uvm_processor_mask_clear(&sysmem->sysmem.mapped_on_phys, gpu->id);
 }
 
 NV_STATUS uvm_mem_translate_gpu_attributes(const UvmGpuMappingAttributes *attrs,
@@ -288,15 +290,15 @@ uvm_chunk_sizes_mask_t uvm_mem_kernel_chunk_sizes(uvm_gpu_t *gpu)
     // Get the mmu mode hal directly as the internal address space tree has not
     // been created yet.
     uvm_mmu_mode_hal_t *hal = gpu->parent->arch_hal->mmu_mode_hal(gpu->big_page.internal_size);
-    NvU32 page_sizes = hal->page_sizes();
+    NvU64 page_sizes = hal->page_sizes();
 
     return (uvm_chunk_sizes_mask_t)(page_sizes & UVM_CHUNK_SIZES_MASK);
 }
 
-static NvU32 mem_pick_chunk_size(uvm_mem_t *mem)
+static NvU64 mem_pick_chunk_size(uvm_mem_t *mem)
 {
-    NvU32 biggest_page_size;
-    NvU32 chunk_size;
+    NvU64 biggest_page_size;
+    NvU64 chunk_size;
 
     if (uvm_mem_is_sysmem(mem))
         return PAGE_SIZE;
@@ -312,13 +314,13 @@ static NvU32 mem_pick_chunk_size(uvm_mem_t *mem)
 
     // When UVM_PAGE_SIZE_DEFAULT is used on NUMA-enabled GPUs, we force
     // chunk_size to be PAGE_SIZE at least, to allow CPU mappings.
-    if (mem->backing_gpu->parent->numa_info.enabled)
-        chunk_size = max(chunk_size, (NvU32)PAGE_SIZE);
+    if (mem->backing_gpu->mem_info.numa.enabled)
+        chunk_size = max(chunk_size, (NvU64)PAGE_SIZE);
 
     return chunk_size;
 }
 
-static NvU32 mem_pick_gpu_page_size(uvm_mem_t *mem, uvm_gpu_t *gpu, uvm_page_tree_t *gpu_page_tree)
+static NvU64 mem_pick_gpu_page_size(uvm_mem_t *mem, uvm_gpu_t *gpu, uvm_page_tree_t *gpu_page_tree)
 {
     if (uvm_mem_is_vidmem(mem)) {
         // For vidmem allocations the chunk size is picked out of the supported
@@ -364,7 +366,7 @@ static void mem_free_sysmem_dma_chunks(uvm_mem_t *mem)
     NvU32 gpu_index;
 
     UVM_ASSERT(uvm_mem_is_sysmem_dma(mem));
-    gpu_index = uvm_global_id_gpu_index(mem->dma_owner->global_id);
+    gpu_index = uvm_id_gpu_index(mem->dma_owner->id);
 
     if (!mem->sysmem.pages || !mem->sysmem.va)
         goto end;
@@ -373,9 +375,9 @@ static void mem_free_sysmem_dma_chunks(uvm_mem_t *mem)
         if (!mem->sysmem.va[i])
             break;
 
-        uvm_gpu_dma_free_page(mem->dma_owner->parent,
-                              mem->sysmem.va[i],
-                              mem->sysmem.dma_addrs[gpu_index][i]);
+        uvm_parent_gpu_dma_free_page(mem->dma_owner->parent,
+                                     mem->sysmem.va[i],
+                                     mem->sysmem.dma_addrs[gpu_index][i]);
     }
 
 end:
@@ -423,7 +425,7 @@ static void mem_free_chunks(uvm_mem_t *mem)
 static NV_STATUS mem_alloc_dma_addrs(uvm_mem_t *mem, const uvm_gpu_t *gpu)
 {
     NvU64 *dma_addrs = NULL;
-    NvU32 gpu_index = uvm_global_id_gpu_index(gpu->global_id);
+    NvU32 gpu_index = uvm_id_gpu_index(gpu->id);
 
     dma_addrs = uvm_kvmalloc_zero(sizeof(*dma_addrs) * mem->chunks_count);
     if (!dma_addrs)
@@ -449,6 +451,11 @@ static gfp_t sysmem_allocation_gfp_flags(int order, bool zero)
     return gfp_flags;
 }
 
+
+// This allocation is a non-protected memory allocation under Confidential
+// Computing.
+//
+//
 // There is a tighter coupling between allocation and mapping because of the
 // allocator UVM must use. Hence, this function does the equivalent of
 // uvm_mem_map_gpu_phys().
@@ -462,7 +469,7 @@ static NV_STATUS mem_alloc_sysmem_dma_chunks(uvm_mem_t *mem, gfp_t gfp_flags)
     NvU64 *dma_addrs;
 
     UVM_ASSERT_MSG(mem->chunk_size == PAGE_SIZE,
-                   "mem->chunk_size is 0x%x. PAGE_SIZE is only supported.",
+                   "mem->chunk_size is 0x%llx. PAGE_SIZE is only supported.",
                    mem->chunk_size);
     UVM_ASSERT(uvm_mem_is_sysmem_dma(mem));
 
@@ -475,10 +482,10 @@ static NV_STATUS mem_alloc_sysmem_dma_chunks(uvm_mem_t *mem, gfp_t gfp_flags)
     if (status != NV_OK)
         goto error;
 
-    dma_addrs = mem->sysmem.dma_addrs[uvm_global_id_gpu_index(mem->dma_owner->global_id)];
+    dma_addrs = mem->sysmem.dma_addrs[uvm_id_gpu_index(mem->dma_owner->id)];
 
     for (i = 0; i < mem->chunks_count; ++i) {
-        mem->sysmem.va[i] = uvm_gpu_dma_alloc_page(mem->dma_owner->parent, gfp_flags, &dma_addrs[i]);
+        mem->sysmem.va[i] = uvm_parent_gpu_dma_alloc_page(mem->dma_owner->parent, gfp_flags, &dma_addrs[i]);
         if (!mem->sysmem.va[i])
             goto err_no_mem;
 
@@ -523,7 +530,7 @@ static NV_STATUS mem_alloc_sysmem_chunks(uvm_mem_t *mem, gfp_t gfp_flags)
 
 // In case of failure, the caller is required to handle cleanup by calling
 // uvm_mem_free
-static NV_STATUS mem_alloc_vidmem_chunks(uvm_mem_t *mem, bool zero, bool is_protected)
+static NV_STATUS mem_alloc_vidmem_chunks(uvm_mem_t *mem, bool zero)
 {
     NV_STATUS status;
 
@@ -548,8 +555,9 @@ static NV_STATUS mem_alloc_vidmem_chunks(uvm_mem_t *mem, bool zero, bool is_prot
                                       UVM_PMM_ALLOC_FLAGS_NONE,
                                       mem->vidmem.chunks,
                                       NULL);
+
     if (status != NV_OK) {
-        UVM_ERR_PRINT("pmm_gpu_alloc(count=%zd, size=0x%x) failed: %s\n",
+        UVM_ERR_PRINT("uvm_pmm_gpu_alloc_kernel (count=%zd, size=0x%llx) failed: %s\n",
                       mem->chunks_count,
                       mem->chunk_size,
                       nvstatusToString(status));
@@ -559,7 +567,7 @@ static NV_STATUS mem_alloc_vidmem_chunks(uvm_mem_t *mem, bool zero, bool is_prot
     return NV_OK;
 }
 
-static NV_STATUS mem_alloc_chunks(uvm_mem_t *mem, struct mm_struct *mm, bool zero, bool is_protected)
+static NV_STATUS mem_alloc_chunks(uvm_mem_t *mem, struct mm_struct *mm, bool zero)
 {
     if (uvm_mem_is_sysmem(mem)) {
         gfp_t gfp_flags;
@@ -581,10 +589,10 @@ static NV_STATUS mem_alloc_chunks(uvm_mem_t *mem, struct mm_struct *mm, bool zer
         return status;
     }
 
-    return mem_alloc_vidmem_chunks(mem, zero, is_protected);
+    return mem_alloc_vidmem_chunks(mem, zero);
 }
 
-NV_STATUS uvm_mem_map_kernel(uvm_mem_t *mem, const uvm_global_processor_mask_t *mask)
+NV_STATUS uvm_mem_map_kernel(uvm_mem_t *mem, const uvm_processor_mask_t *mask)
 {
     uvm_gpu_t *gpu;
     NV_STATUS status;
@@ -592,13 +600,13 @@ NV_STATUS uvm_mem_map_kernel(uvm_mem_t *mem, const uvm_global_processor_mask_t *
     if (!mask)
         return NV_OK;
 
-    if (uvm_global_processor_mask_test(mask, UVM_GLOBAL_ID_CPU)) {
+    if (uvm_processor_mask_test(mask, UVM_ID_CPU)) {
         status = uvm_mem_map_cpu_kernel(mem);
         if (status != NV_OK)
             return status;
     }
 
-    for_each_global_gpu_in_mask(gpu, mask) {
+    for_each_gpu_in_mask(gpu, mask) {
         status = uvm_mem_map_gpu_kernel(mem, gpu);
         if (status != NV_OK)
             return status;
@@ -611,7 +619,6 @@ NV_STATUS uvm_mem_alloc(const uvm_mem_alloc_params_t *params, uvm_mem_t **mem_ou
     NV_STATUS status;
     NvU64 physical_size;
     uvm_mem_t *mem = NULL;
-    bool is_protected = false;
 
     UVM_ASSERT(params->size > 0);
 
@@ -633,7 +640,7 @@ NV_STATUS uvm_mem_alloc(const uvm_mem_alloc_params_t *params, uvm_mem_t **mem_ou
     physical_size = UVM_ALIGN_UP(mem->size, mem->chunk_size);
     mem->chunks_count = physical_size / mem->chunk_size;
 
-    status = mem_alloc_chunks(mem, params->mm, params->zero, is_protected);
+    status = mem_alloc_chunks(mem, params->mm, params->zero);
     if (status != NV_OK)
         goto error;
 
@@ -675,7 +682,7 @@ static void mem_deinit_user_mapping(uvm_mem_t *mem)
     if (mem->user == NULL)
         return;
 
-    if (!uvm_global_processor_mask_empty(&mem->user->mapped_on))
+    if (!uvm_processor_mask_empty(&mem->user->mapped_on))
         return;
 
     uvm_kvfree(mem->user);
@@ -703,7 +710,7 @@ static NV_STATUS mem_map_cpu_to_sysmem_kernel(uvm_mem_t *mem)
 {
     struct page **pages = mem->sysmem.pages;
     size_t num_pages = uvm_mem_physical_size(mem) / PAGE_SIZE;
-    pgprot_t prot = PAGE_KERNEL;
+    pgprot_t prot;
 
     UVM_ASSERT(uvm_mem_is_sysmem(mem));
 
@@ -718,8 +725,10 @@ static NV_STATUS mem_map_cpu_to_sysmem_kernel(uvm_mem_t *mem)
             pages[page_index] = mem_cpu_page(mem, page_index * PAGE_SIZE);
     }
 
-    if (g_uvm_global.sev_enabled)
-        prot = PAGE_KERNEL_NOENC;
+    if (g_uvm_global.conf_computing_enabled && uvm_mem_is_sysmem_dma(mem))
+        prot = uvm_pgprot_decrypted(PAGE_KERNEL_NOENC);
+    else
+        prot = PAGE_KERNEL;
 
     mem->kernel.cpu_addr = vmap(pages, num_pages, VM_MAP, prot);
 
@@ -874,7 +883,7 @@ NV_STATUS uvm_mem_map_cpu_kernel(uvm_mem_t *mem)
 
 static void sysmem_unmap_gpu_phys(uvm_mem_t *mem, uvm_gpu_t *gpu)
 {
-    NvU64 *dma_addrs = mem->sysmem.dma_addrs[uvm_global_id_gpu_index(gpu->global_id)];
+    NvU64 *dma_addrs = mem->sysmem.dma_addrs[uvm_id_gpu_index(gpu->id)];
     NvU32 i;
 
     UVM_ASSERT(uvm_mem_is_sysmem(mem));
@@ -887,12 +896,12 @@ static void sysmem_unmap_gpu_phys(uvm_mem_t *mem, uvm_gpu_t *gpu)
             // partial map_gpu_sysmem_iommu() operation.
             break;
         }
-        uvm_gpu_unmap_cpu_pages(gpu->parent, dma_addrs[i], mem->chunk_size);
+        uvm_parent_gpu_unmap_cpu_pages(gpu->parent, dma_addrs[i], mem->chunk_size);
         dma_addrs[i] = 0;
     }
 
     uvm_kvfree(dma_addrs);
-    mem->sysmem.dma_addrs[uvm_global_id_gpu_index(gpu->global_id)] = NULL;
+    mem->sysmem.dma_addrs[uvm_id_gpu_index(gpu->id)] = NULL;
 }
 
 static NV_STATUS sysmem_map_gpu_phys(uvm_mem_t *mem, uvm_gpu_t *gpu)
@@ -908,10 +917,10 @@ static NV_STATUS sysmem_map_gpu_phys(uvm_mem_t *mem, uvm_gpu_t *gpu)
         return status;
 
     for (i = 0; i < mem->chunks_count; ++i) {
-        status = uvm_gpu_map_cpu_pages(gpu->parent,
-                                       mem->sysmem.pages[i],
-                                       mem->chunk_size,
-                                       &mem->sysmem.dma_addrs[uvm_global_id_gpu_index(gpu->global_id)][i]);
+        status = uvm_parent_gpu_map_cpu_pages(gpu->parent,
+                                              mem->sysmem.pages[i],
+                                              mem->chunk_size,
+                                              &mem->sysmem.dma_addrs[uvm_id_gpu_index(gpu->id)][i]);
         if (status != NV_OK)
             goto error;
     }
@@ -943,7 +952,7 @@ static uvm_gpu_phys_address_t mem_gpu_physical_vidmem(uvm_mem_t *mem, size_t off
 
 static uvm_gpu_phys_address_t mem_gpu_physical_sysmem(uvm_mem_t *mem, uvm_gpu_t *gpu, size_t offset)
 {
-    NvU64 *dma_addrs = mem->sysmem.dma_addrs[uvm_global_id_gpu_index(gpu->global_id)];
+    NvU64 *dma_addrs = mem->sysmem.dma_addrs[uvm_id_gpu_index(gpu->id)];
     NvU64 dma_addr = dma_addrs[offset / mem->chunk_size];
 
     UVM_ASSERT(uvm_mem_is_sysmem(mem));
@@ -982,12 +991,12 @@ uvm_gpu_address_t uvm_mem_gpu_address_copy(uvm_mem_t *mem, uvm_gpu_t *accessing_
     UVM_ASSERT(uvm_mem_is_physically_contiguous(mem, offset, size));
 
     if (uvm_mem_is_sysmem(mem) || uvm_mem_is_local_vidmem(mem, accessing_gpu))
-        return uvm_mem_gpu_address_physical(mem, accessing_gpu, offset, size);
+        return uvm_gpu_address_copy(accessing_gpu, uvm_mem_gpu_physical(mem, accessing_gpu, offset, size));
 
     // Peer GPUs may need to use some form of translation (identity mappings,
     // indirect peers) to copy.
     chunk = mem_get_chunk(mem, offset, &chunk_offset);
-    copy_addr = uvm_pmm_gpu_peer_copy_address(&mem->backing_gpu->pmm, chunk, accessing_gpu);
+    copy_addr = uvm_gpu_peer_copy_address(mem->backing_gpu, chunk->address, accessing_gpu);
     copy_addr.address += chunk_offset;
     return copy_addr;
 }
@@ -1030,7 +1039,7 @@ static NV_STATUS mem_map_gpu(uvm_mem_t *mem,
                              uvm_page_table_range_vec_t **range_vec)
 {
     NV_STATUS status;
-    NvU32 page_size;
+    NvU64 page_size;
     uvm_pmm_alloc_flags_t pmm_flags = UVM_PMM_ALLOC_FLAGS_EVICT;
 
     uvm_mem_pte_maker_data_t pte_maker_data = {
@@ -1039,7 +1048,15 @@ static NV_STATUS mem_map_gpu(uvm_mem_t *mem,
         };
 
     page_size = mem_pick_gpu_page_size(mem, gpu, tree);
-    UVM_ASSERT_MSG(uvm_mmu_page_size_supported(tree, page_size), "page_size 0x%x\n", page_size);
+    UVM_ASSERT_MSG(uvm_mmu_page_size_supported(tree, page_size), "page_size 0x%llx\n", page_size);
+
+    // When the Confidential Computing feature is enabled, DMA allocations are
+    // majoritarily allocated and managed by a per-GPU DMA buffer pool
+    // (uvm_conf_computing_dma_buffer_pool_t). Because we would typically
+    // already hold the DMA_BUFFER_POOL lock at this time, we cannot hold
+    // the block lock. Allocate PTEs without eviction in this context.
+    if (uvm_mem_is_sysmem_dma(mem))
+        pmm_flags = UVM_PMM_ALLOC_FLAGS_NONE;
 
     status = uvm_page_table_range_vec_create(tree,
                                              gpu_va,
@@ -1091,7 +1108,7 @@ static void mem_deinit_gpu_kernel_range(uvm_mem_t *mem)
 
     // Do not remove the range allocation if there is any GPU where the memory
     // is still mapped on kernel space.
-    if (UVM_GLOBAL_ID_IS_VALID(uvm_global_processor_mask_find_first_gpu_id(&mem->kernel.mapped_on)))
+    if (UVM_ID_IS_VALID(uvm_processor_mask_find_first_gpu_id(&mem->kernel.mapped_on)))
         return;
 
     uvm_range_allocator_free(&g_free_ranges, &mem->kernel.range_alloc);
@@ -1121,7 +1138,7 @@ NV_STATUS uvm_mem_map_gpu_kernel(uvm_mem_t *mem, uvm_gpu_t *gpu)
         return status;
 
     gpu_va = reserved_gpu_va(mem, gpu);
-    range_vec = &mem->kernel.range_vecs[uvm_global_id_gpu_index(gpu->global_id)];
+    range_vec = &mem->kernel.range_vecs[uvm_id_gpu_index(gpu->id)];
 
     status = mem_map_gpu(mem, gpu, gpu_va, &gpu->address_space_tree, &attrs, range_vec);
     if (status != NV_OK)
@@ -1167,7 +1184,7 @@ NV_STATUS uvm_mem_map_gpu_user(uvm_mem_t *mem,
         return status;
 
     gpu_va_space = uvm_gpu_va_space_get(mem->user->va_space, gpu);
-    range_vec = &mem->user->range_vecs[uvm_global_id_gpu_index(gpu->global_id)];
+    range_vec = &mem->user->range_vecs[uvm_id_gpu_index(gpu->id)];
 
     status = mem_map_gpu(mem, gpu, gpu_va, &gpu_va_space->page_tables, attrs, range_vec);
     if (status != NV_OK)
@@ -1187,7 +1204,7 @@ void uvm_mem_unmap_gpu_user(uvm_mem_t *mem, uvm_gpu_t *gpu)
     if (!uvm_mem_mapped_on_gpu_user(mem, gpu))
         return;
 
-    mem_unmap_gpu(mem, gpu, &mem->user->range_vecs[uvm_global_id_gpu_index(gpu->global_id)]);
+    mem_unmap_gpu(mem, gpu, &mem->user->range_vecs[uvm_id_gpu_index(gpu->id)]);
     mem_clear_mapped_on_gpu_user(mem, gpu);
     mem_deinit_user_mapping(mem);
 }
@@ -1197,7 +1214,7 @@ void uvm_mem_unmap_gpu_kernel(uvm_mem_t *mem, uvm_gpu_t *gpu)
     if (!uvm_mem_mapped_on_gpu_kernel(mem, gpu))
         return;
 
-    mem_unmap_gpu(mem, gpu, &mem->kernel.range_vecs[uvm_global_id_gpu_index(gpu->global_id)]);
+    mem_unmap_gpu(mem, gpu, &mem->kernel.range_vecs[uvm_id_gpu_index(gpu->id)]);
     mem_clear_mapped_on_gpu_kernel(mem, gpu);
     mem_deinit_gpu_kernel_range(mem);
 }
@@ -1205,7 +1222,7 @@ void uvm_mem_unmap_gpu_kernel(uvm_mem_t *mem, uvm_gpu_t *gpu)
 static bool mem_can_be_phys_mapped_on_gpu(uvm_mem_t *mem, uvm_gpu_t *gpu)
 {
     if (uvm_mem_is_sysmem(mem))
-        return sysmem_can_be_mapped(mem);
+        return sysmem_can_be_mapped_on_gpu(mem);
     else
         return uvm_mem_is_local_vidmem(mem, gpu);
 }
@@ -1267,7 +1284,7 @@ void uvm_mem_free(uvm_mem_t *mem)
     uvm_mem_unmap_cpu_kernel(mem);
 
     if (mem->user != NULL) {
-        for_each_global_gpu_in_mask(gpu, &mem->user->mapped_on) {
+        for_each_gpu_in_mask(gpu, &mem->user->mapped_on) {
             uvm_mem_unmap_gpu_user(mem, gpu);
 
             // If we unmapped the last device, the user mapping is freed, so
@@ -1277,11 +1294,11 @@ void uvm_mem_free(uvm_mem_t *mem)
         }
     }
 
-    for_each_global_gpu_in_mask(gpu, &mem->kernel.mapped_on)
+    for_each_gpu_in_mask(gpu, &mem->kernel.mapped_on)
         uvm_mem_unmap_gpu_kernel(mem, gpu);
 
     if (uvm_mem_is_sysmem(mem)) {
-        for_each_global_gpu_in_mask(gpu, &mem->sysmem.mapped_on_phys)
+        for_each_gpu_in_mask(gpu, &mem->sysmem.mapped_on_phys)
             uvm_mem_unmap_gpu_phys(mem, gpu);
     }
 
@@ -1306,10 +1323,20 @@ NvU64 uvm_mem_get_gpu_va_kernel(uvm_mem_t *mem, uvm_gpu_t *gpu)
 
 uvm_gpu_address_t uvm_mem_gpu_address_virtual_kernel(uvm_mem_t *mem, uvm_gpu_t *gpu)
 {
-    return uvm_gpu_address_virtual(uvm_mem_get_gpu_va_kernel(mem, gpu));
+    uvm_gpu_address_t addr = uvm_gpu_address_virtual(uvm_mem_get_gpu_va_kernel(mem, gpu));
+
+    if (g_uvm_global.conf_computing_enabled && mem->dma_owner)
+        addr.is_unprotected = true;
+
+    return addr;
 }
 
 uvm_gpu_address_t uvm_mem_gpu_address_physical(uvm_mem_t *mem, uvm_gpu_t *gpu, NvU64 offset, NvU64 size)
 {
-    return uvm_gpu_address_from_phys(uvm_mem_gpu_physical(mem, gpu, offset, size));
+    uvm_gpu_address_t addr = uvm_gpu_address_from_phys(uvm_mem_gpu_physical(mem, gpu, offset, size));
+
+    if (g_uvm_global.conf_computing_enabled && mem->dma_owner)
+        addr.is_unprotected = true;
+
+    return addr;
 }
