@@ -108,6 +108,107 @@ typedef volatile struct _clc673_tag0 {
 #define NVC673_SYS_CAP_DSI_EXISTS__SIZE_1                                      4 /*       */
 #define NVC673_SYS_CAP_DSI_EXISTS_NO                                  0x00000000 /* RW--V */
 #define NVC673_SYS_CAP_DSI_EXISTS_YES                                 0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB                                                0x4 /* RW-4R */
+#define NVC673_SYS_CAPB_WINDOW0_EXISTS                                        0:0 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW0_EXISTS_NO                              0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW0_EXISTS_YES                             0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW1_EXISTS                                        1:1 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW1_EXISTS_NO                              0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW1_EXISTS_YES                             0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW2_EXISTS                                        2:2 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW2_EXISTS_NO                              0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW2_EXISTS_YES                             0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW3_EXISTS                                        3:3 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW3_EXISTS_NO                              0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW3_EXISTS_YES                             0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW4_EXISTS                                        4:4 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW4_EXISTS_NO                              0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW4_EXISTS_YES                             0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW5_EXISTS                                        5:5 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW5_EXISTS_NO                              0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW5_EXISTS_YES                             0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW6_EXISTS                                        6:6 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW6_EXISTS_NO                              0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW6_EXISTS_YES                             0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW7_EXISTS                                        7:7 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW7_EXISTS_NO                              0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW7_EXISTS_YES                             0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW8_EXISTS                                        8:8 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW8_EXISTS_NO                              0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW8_EXISTS_YES                             0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW9_EXISTS                                        9:9 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW9_EXISTS_NO                              0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW9_EXISTS_YES                             0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW10_EXISTS                                     10:10 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW10_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW10_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW11_EXISTS                                     11:11 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW11_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW11_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW12_EXISTS                                     12:12 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW12_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW12_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW13_EXISTS                                     13:13 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW13_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW13_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW14_EXISTS                                     14:14 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW14_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW14_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW15_EXISTS                                     15:15 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW15_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW15_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW16_EXISTS                                     16:16 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW16_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW16_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW17_EXISTS                                     17:17 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW17_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW17_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW18_EXISTS                                     18:18 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW18_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW18_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW19_EXISTS                                     19:19 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW19_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW19_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW20_EXISTS                                     20:20 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW20_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW20_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW21_EXISTS                                     21:21 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW21_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW21_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW22_EXISTS                                     22:22 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW22_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW22_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW23_EXISTS                                     23:23 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW23_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW23_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW24_EXISTS                                     24:24 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW24_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW24_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW25_EXISTS                                     25:25 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW25_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW25_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW26_EXISTS                                     26:26 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW26_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW26_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW27_EXISTS                                     27:27 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW27_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW27_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW28_EXISTS                                     28:28 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW28_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW28_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW29_EXISTS                                     29:29 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW29_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW29_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW30_EXISTS                                     30:30 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW30_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW30_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW31_EXISTS                                     31:31 /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW31_EXISTS_NO                             0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW31_EXISTS_YES                            0x00000001 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW_EXISTS(i)                          (0+(i)):(0+(i)) /* RWIVF */
+#define NVC673_SYS_CAPB_WINDOW_EXISTS__SIZE_1                                  32 /*       */
+#define NVC673_SYS_CAPB_WINDOW_EXISTS_NO                               0x00000000 /* RW--V */
+#define NVC673_SYS_CAPB_WINDOW_EXISTS_YES                              0x00000001 /* RW--V */
 #define NVC673_IHUB_COMMON_CAPA                                       0x10 /* RW-4R */
 #define NVC673_IHUB_COMMON_CAPA_MEMPOOL_ENTRIES                             15:0 /* RWIUF */
 #define NVC673_IHUB_COMMON_CAPA_MEMPOOL_ENTRY_WIDTH                        17:16 /* RWIVF */
@@ -392,6 +493,11 @@ typedef volatile struct _clc673_tag0 {
 #define NVC673_PRECOMP_WIN_PIPE_HDR_CAPF__SIZE_1                                          32 /*       */
 #define NVC673_PRECOMP_WIN_PIPE_HDR_CAPF_VSCLR_MAX_PIXELS_2TAP                          15:0 /* RWIVF */
 #define NVC673_PRECOMP_WIN_PIPE_HDR_CAPF_VSCLR_MAX_PIXELS_5TAP                         31:16 /* RWIVF */
+#define NVC673_SOR_CLK_CAP(i)                                 (0x608+(i)*4) /* RW-4A */
+#define NVC673_SOR_CLK_CAP__SIZE_1                                             8 /*       */
+#define NVC673_SOR_CLK_CAP_DP_MAX                                            7:0 /* RWIUF */
+#define NVC673_SOR_CLK_CAP_TMDS_MAX                                        23:16 /* RWIUF */
+#define NVC673_SOR_CLK_CAP_LVDS_MAX                                        31:24 /* RWIUF */
 
 #ifdef __cplusplus
 };
