@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2012-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2012-2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -71,9 +71,10 @@ osCreateMemFromOsDescriptor
     NV_STATUS rmStatus;
     void *pPrivate;
 
+    pClient = serverutilGetClientUnderLock(hClient);
     if ((pDescriptor == NvP64_NULL) ||
         (*pLimit == 0) ||
-        (serverutilGetClientUnderLock(hClient, &pClient) != NV_OK))
+        (pClient == NULL))
     {
         return NV_ERR_INVALID_PARAM_STRUCT;
     }
@@ -361,6 +362,23 @@ osCheckGpuBarsOverlapAddrRange
     return NV_OK;
 }
 
+static NvU64
+_doWarBug4040336
+(
+    OBJGPU *pGpu,
+    NvU64 addr
+)
+{
+    if (gpuIsWarBug4040336Enabled(pGpu))
+    {
+        if ((addr & 0xffffffff00000000ULL) == 0x7fff00000000ULL)
+        {
+            addr = addr & 0xffffffffULL;
+        }
+    }
+    return addr;
+}
+
 static NV_STATUS
 osCreateOsDescriptorFromIoMemory
 (
@@ -439,6 +457,14 @@ osCreateOsDescriptorFromIoMemory
         return rmStatus;
     }
 
+    //
+    // BF3's PCIe MMIO bus address at 0x800000000000(CPU PA 0x7fff00000000) is
+    // too high for Ampere to address. As a result, BF3's bus address is
+    // moved to < 4GB. Now, the CPU PA and the bus address are no longer 1:1
+    // and needs to be adjusted.
+    //
+    *base = _doWarBug4040336(pGpu, *base);
+
     rmStatus = memdescCreate(ppMemDesc, pGpu, (*pLimit + 1), 0,
                              NV_MEMORY_CONTIGUOUS, ADDR_SYSMEM,
                              NV_MEMORY_UNCACHED, MEMDESC_FLAGS_NONE);
@@ -521,6 +547,7 @@ osCreateOsDescriptorFromPhysAddr
 )
 {
     NV_STATUS rmStatus;
+    nv_state_t *nv = NV_GET_NV_STATE(pGpu);
     MEMORY_DESCRIPTOR *pMemDesc;
     NvU64 *pPteArray;
     NvU64  base = 0;
@@ -529,6 +556,7 @@ osCreateOsDescriptorFromPhysAddr
     NvU64 *pPhys_addrs;
     NvU64  num_os_pages;
     NvU32  idx;
+
     // Currently only work with contiguous sysmem allocations
     if (!FLD_TEST_DRF(OS02, _FLAGS, _PHYSICALITY, _CONTIGUOUS, flags))
     {
@@ -539,6 +567,15 @@ osCreateOsDescriptorFromPhysAddr
     {
         // Syncpoint memory is uncached, DMA mapping needs to skip CPU sync.
         cache_type = NV_MEMORY_UNCACHED;
+
+        //
+        // Syncpoint memory is NISO. Don't attempt to IOMMU map if the NISO
+        // IOMMU isn't enabled.
+        //
+        if (!NV_SOC_IS_NISO_IOMMU_PRESENT(nv))
+        {
+            memdescFlags |= MEMDESC_FLAGS_SKIP_IOMMU_MAPPING;
+        }
     }
 
     if (FLD_TEST_DRF(OS02, _FLAGS, _ALLOC_NISO_DISPLAY, _YES, flags))
@@ -570,7 +607,7 @@ osCreateOsDescriptorFromPhysAddr
     num_os_pages = NV_RM_PAGES_TO_OS_PAGES(pMemDesc->PageCount);
     pPhys_addrs  = portMemAllocNonPaged(sizeof(NvU64) * num_os_pages);
     if (pPhys_addrs == NULL)
-        return NV_ERR_NO_MEMORY;
+        goto cleanup_memdesc;
 
     for (idx = 0; idx < num_os_pages; idx++) 
     {
@@ -578,20 +615,51 @@ osCreateOsDescriptorFromPhysAddr
     }
 
     *ppPrivate = NULL;
-    rmStatus = nv_register_phys_pages(NV_GET_NV_STATE(pGpu), pPhys_addrs,
-                                      num_os_pages,
+    rmStatus = nv_register_phys_pages(nv, pPhys_addrs, num_os_pages,
                                       memdescGetCpuCacheAttrib(pMemDesc),
                                       ppPrivate);
     if (rmStatus != NV_OK)
+        goto cleanup_memdesc;
+
+    //
+    // For syncpoint memory, if IOMMU skip flag wasn't set earlier,
+    // create IOVA mapping.
+    //
+    if (FLD_TEST_DRF(OS02, _FLAGS, _ALLOC_TYPE_SYNCPOINT, _APERTURE, flags) &&
+        !memdescGetFlag(pMemDesc, MEMDESC_FLAGS_SKIP_IOMMU_MAPPING))
     {
-        memdescDestroy(pMemDesc);
-        goto cleanup;
+        //
+        // memdescMapIommu() requires the OS-private data to be set on the memory
+        // descriptor, but we don't want to wire up the teardown callback just yet:
+        // that callback needs to unpin the pages, but that will already be done
+        // as part of failure handling further up the stack if memdescMapIommu()
+        // fails. So we only set up the priv-data cleanup callback once we're sure
+        // this call will succeed.
+        //
+        memdescSetMemData(pMemDesc, *ppPrivate, NULL);
+
+        rmStatus = memdescMapIommu(pMemDesc, pGpu->busInfo.iovaspaceId);
+        if (rmStatus != NV_OK)
+            goto cleanup_pages;
     }
 
+    // All is well - wire up the cleanup callback now
     memdescSetMemData(pMemDesc, *ppPrivate,
         osDestroyOsDescriptorFromPhysAddr);
 
-cleanup:
+    portMemFree(pPhys_addrs);
+
+    return NV_OK;
+
+cleanup_pages:
+    if (*ppPrivate != NULL)
+    {
+        nv_unregister_phys_pages(NV_GET_NV_STATE(pGpu), *ppPrivate);
+    }
+
+cleanup_memdesc:
+    memdescDestroy(pMemDesc);
+
     portMemFree(pPhys_addrs);
 
     return rmStatus;
@@ -723,7 +791,6 @@ _createMemdescFromDmaBuf
     OBJGPU  *pGpu,
     NvU32    flags,
     nv_dma_buf_t *pImportPriv,
-    void *pUserPages,
     struct sg_table *pImportSgt,
     NvU32 size,
     MEMORY_DESCRIPTOR **ppMemDesc,
@@ -734,12 +801,9 @@ _createMemdescFromDmaBuf
         _createMemdescFromDmaBufSgtHelper(pGpu, flags, pImportPriv, pImportSgt,
                                           size, ppMemDesc, ppPrivate,
                                           osDestroyOsDescriptorFromDmaBuf);
-
-    NV_ASSERT(pUserPages == NULL);
-
     if (rmStatus != NV_OK)
     {
-        nv_dma_release_dma_buf(NULL, pImportPriv);
+        nv_dma_release_dma_buf(pImportPriv);
     }
 
     return rmStatus;
@@ -770,6 +834,23 @@ _createMemdescFromSgt
     return rmStatus;
 }
 
+static nv_dma_device_t *GetDmaDeviceForImport
+(
+    nv_state_t *nv,
+    NvU32 flags
+)
+{
+    if (FLD_TEST_DRF(OS02, _FLAGS, _ALLOC_NISO_DISPLAY, _YES, flags) &&
+        (nv->niso_dma_dev != NULL))
+    {
+        return nv->niso_dma_dev;
+    }
+    else
+    {
+        return nv->dma_dev;
+    }
+}
+
 static NV_STATUS
 osCreateOsDescriptorFromFileHandle
 (
@@ -784,8 +865,8 @@ osCreateOsDescriptorFromFileHandle
 {
     NV_STATUS rmStatus = NV_OK;
     nv_state_t *nv = NV_GET_NV_STATE(pGpu);
+    nv_dma_device_t *dma_dev = NULL;
     NvU32 size = 0;
-    void *pUserPages = NULL;
     nv_dma_buf_t *pImportPriv = NULL;
     struct sg_table *pImportSgt = NULL;
     NvS32 fd;
@@ -799,8 +880,9 @@ osCreateOsDescriptorFromFileHandle
         return NV_ERR_INVALID_ARGUMENT;
     }
 
-    rmStatus = nv_dma_import_from_fd(nv->dma_dev, fd, &size,
-                                     &pUserPages, &pImportSgt, &pImportPriv);
+    dma_dev = GetDmaDeviceForImport(nv, flags);
+    rmStatus = nv_dma_import_from_fd(dma_dev, fd, &size,
+                                     &pImportSgt, &pImportPriv);
     if (rmStatus != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR,
@@ -810,7 +892,7 @@ osCreateOsDescriptorFromFileHandle
     }
 
     return _createMemdescFromDmaBuf(pGpu, flags, pImportPriv,
-                                    pUserPages, pImportSgt,
+                                    pImportSgt,
                                     size, ppMemDesc, ppPrivate);
 }
 
@@ -861,24 +943,24 @@ osCreateOsDescriptorFromDmaBufPtr
 {
     NV_STATUS rmStatus = NV_OK;
     nv_state_t *nv = NV_GET_NV_STATE(pGpu);
+    nv_dma_device_t *dma_dev = NULL;
     NvU32 size = 0;
-    void *pUserPages = NULL;
     nv_dma_buf_t *pImportPriv = NULL;
     struct sg_table *pImportSgt = NULL;
     void *dmaBuf = (void*)((NvUPtr)pDescriptor);
 
-    rmStatus = nv_dma_import_dma_buf(nv->dma_dev, dmaBuf, &size,
-                                     &pUserPages, &pImportSgt, &pImportPriv);
+    dma_dev = GetDmaDeviceForImport(nv, flags);
+    rmStatus = nv_dma_import_dma_buf(dma_dev, dmaBuf, &size,
+                                     &pImportSgt, &pImportPriv);
     if (rmStatus != NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR,
-                  "%s(): Error (%d) while trying to import dma_buf!\n",
-                  __FUNCTION__, rmStatus);
+        NV_PRINTF_COND(rmStatus == NV_ERR_NOT_SUPPORTED, LEVEL_INFO, LEVEL_ERROR,
+                       "Error (%d) while trying to import dma_buf!\n", rmStatus);
         return rmStatus;
     }
 
     return _createMemdescFromDmaBuf(pGpu, flags, pImportPriv,
-                                    pUserPages, pImportSgt,
+                                    pImportSgt,
                                     size, ppMemDesc, ppPrivate);
 }
 
@@ -948,14 +1030,22 @@ osDestroyOsDescriptorPageArray
     nv_unregister_user_pages(NV_GET_NV_STATE(pGpu), osPageCount,
                              NULL /* import_priv */, &pPrivate);
 
-    if (memdescGetFlag(pMemDesc, MEMDESC_FLAGS_FOREIGN_PAGE) == NV_FALSE)
+    if (memdescGetFlag(pMemDesc, MEMDESC_FLAGS_FOREIGN_PAGE))
     {
-        status = os_unlock_user_pages(osPageCount, pPrivate);
-        NV_ASSERT(status == NV_OK);
+        os_free_mem(pPrivate);
     }
     else
     {
-        os_free_mem(pPrivate);
+        //
+        // We use MEMDESC_FLAGS_USER_READ_ONLY because this reflects the
+        // NVOS02_FLAGS_ALLOC_USER_READ_ONLY flag value passed into
+        // os_lock_user_pages(). That flag also results in
+        // MEMDESC_FLAGS_DEVICE_READ_ONLY being set.
+        //
+        NvBool writable = !memdescGetFlag(pMemDesc, MEMDESC_FLAGS_USER_READ_ONLY);
+        NvU32 flags = DRF_NUM(_LOCK_USER_PAGES, _FLAGS, _WRITE, writable);
+        status = os_unlock_user_pages(osPageCount, pPrivate, flags);
+        NV_ASSERT(status == NV_OK);
     }
 }
 
@@ -993,7 +1083,7 @@ osDestroyOsDescriptorFromDmaBuf
      * SGT.
      */
 
-    nv_dma_release_dma_buf(NULL, pImportPriv);
+    nv_dma_release_dma_buf(pImportPriv);
 }
 
 static void
