@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2013-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2013-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -73,9 +73,27 @@ kbifConstructEngine_IMPL
     // Disables P2P on VF
     kbifDisableP2PTransactions_HAL(pGpu, pKernelBif);
 
+    // Cache MNOC interface support
+    kbifCacheMnocSupport_HAL(pGpu, pKernelBif);
+
+    // Cache FLR support
+    kbifCacheFlrSupport_HAL(pGpu, pKernelBif);
+
+    // Cache 64B BAR0 support
+    kbifCache64bBar0Support_HAL(pGpu, pKernelBif);
+
+    // Cache VF info
+    kbifCacheVFInfo_HAL(pGpu, pKernelBif);
+
+    // Cache GPU link capabilities
+    kbifGetGpuLinkCapabilities(pGpu, pKernelBif);
+
     // Used to track when the link has gone into Recovery, which can cause CEs.
     pKernelBif->EnteredRecoverySinceErrorsLastChecked = NV_FALSE;
 
+    // Default scale is 1 and could be overriden by registry
+    pKernelBif->flrDevInitTimeoutScale = 1;
+
     return NV_OK;
 }
 
@@ -95,7 +113,12 @@ kbifStateInitLocked_IMPL
     OBJSYS    *pSys   = SYS_GET_INSTANCE();
     OBJOS     *pOS    = SYS_GET_OS(pSys);
     OBJCL     *pCl    = SYS_GET_CL(pSys);
-    NV_STATUS  status = NV_OK;
+
+    kbifInitXveRegMap_HAL(pGpu, pKernelBif, 1);
+    kbifInitXveRegMap_HAL(pGpu, pKernelBif, 0);
+
+    // Do the HAL dependent init
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, kbifInit_HAL(pGpu, pKernelBif));
 
     // Return early if GPU is connected to an unsupported chipset
     if (pCl->getProperty(pCl, PDB_PROP_CL_UNSUPPORTED_CHIPSET))
@@ -104,11 +127,7 @@ kbifStateInitLocked_IMPL
     }
 
     // Initialize OS mapping and core logic
-    status = osInitMapping(pGpu);
-    if (status != NV_OK)
-    {
-        return status;
-    }
+    NV_CHECK_OK_OR_RETURN(LEVEL_ERROR, osInitMapping(pGpu));
 
     // Initialize BIF static info
     kbifStaticInfoInit(pGpu, pKernelBif);
@@ -127,7 +146,7 @@ kbifStateInitLocked_IMPL
         pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_SUPPORT_NONCOHERENT, NV_FALSE);
     }
 
-    return status;
+    return NV_OK;
 }
 
 /*!
@@ -153,6 +172,15 @@ kbifStateLoad_IMPL
     // Check for stale PCI-E dev ctrl/status errors and AER errors
     kbifClearConfigErrors(pGpu, pKernelBif, NV_TRUE, KBIF_CLEAR_XVE_AER_ALL_MASK);
 
+    // Initialize LTR from config space
+    kbifInitLtr_HAL(pGpu, pKernelBif);
+
+    //  Cache PCI config registers to be restored during resume
+    if (!pGpu->getProperty(pGpu, PDB_PROP_GPU_IN_PM_RESUME_CODEPATH))
+    {
+        kbifSavePcieConfigRegisters_HAL(pGpu, pKernelBif);
+    }
+
     //
     // A vGPU cannot disappear and these accesses are
     // particularly expensive on vGPUs
@@ -160,7 +188,7 @@ kbifStateLoad_IMPL
     if (pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_CHECK_IF_GPU_EXISTS_DEF) &&
         !IS_VIRTUAL(pGpu))
     {
-        osSchedule1SecondCallback(pGpu, _kbifCheckIfGpuExists, NULL, NV_OS_1HZ_REPEAT);
+        osSchedule1HzCallback(pGpu, _kbifCheckIfGpuExists, NULL, NV_OS_1HZ_REPEAT);
     }
 
     return NV_OK;
@@ -245,9 +273,68 @@ kbifStateUnload_IMPL
     NvU32      flags
 )
 {
+
     return NV_OK;
 }
 
+
+/*!
+ * @brief Cache L1substates info from chipset or bridge behind the GPUs.
+ *
+ * @param[in]  pGpu       GPU object pointer
+ * @param[in]  pKernelBif Kernel BIF object pointer
+ */
+void
+kbifCacheChipsetL1SubstatesEnable_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    OBJSYS                  *pSys = SYS_GET_INSTANCE();
+    OBJCL                   *pCl  = SYS_GET_CL(pSys);
+    PexL1SubstateCapability  L1SsCap;
+    NvBool                   bChipsetPcipmL12Enabled;
+    NvBool                   bChipsetPcipmL11Enabled;
+    NvBool                   bChipsetAspmL12Enabled;
+    NvBool                   bChipsetAspmL11Enabled;
+
+    if (pGpu->gpuClData.upstreamPort.addr.valid)
+    {
+        if (pCl && (clPcieReadL1SsCapability(pGpu, pCl, &L1SsCap) == NV_OK))
+        {
+            pKernelBif->chipsetL1ssEnable = L1SsCap.Control1Reg;
+
+            bChipsetPcipmL12Enabled = FLD_TEST_DRF(2080, _CTRL_BUS_INFO_PCIE_L1_SS_CTRL1, _PCIPM_L1_2_ENABLED, _YES, L1SsCap.Control1Reg);
+            bChipsetPcipmL11Enabled = FLD_TEST_DRF(2080, _CTRL_BUS_INFO_PCIE_L1_SS_CTRL1, _PCIPM_L1_1_ENABLED, _YES, L1SsCap.Control1Reg);
+            bChipsetAspmL12Enabled  = FLD_TEST_DRF(2080, _CTRL_BUS_INFO_PCIE_L1_SS_CTRL1, _ASPM_L1_2_ENABLED,  _YES, L1SsCap.Control1Reg);
+            bChipsetAspmL11Enabled  = FLD_TEST_DRF(2080, _CTRL_BUS_INFO_PCIE_L1_SS_CTRL1, _ASPM_L1_1_ENABLED,  _YES, L1SsCap.Control1Reg);
+
+            //
+            // L1 PM substates can be done with ASPM or PCI-PM
+            // Its supported if any of L1.1/L1.2 PCI-PM/ASPM susbtates is enabled
+            //
+            if (bChipsetPcipmL12Enabled || bChipsetPcipmL11Enabled || bChipsetAspmL12Enabled || bChipsetAspmL11Enabled)
+            {
+                // Chipset Supports L1 PM Substates
+                NV_PRINTF(LEVEL_INFO,
+                          "Chipset supports L1 PM substates.\n      L1 PM Capabilities Register 0x%x\n      L1 PM Control 1 Register 0x%x\n      L1 PM Control 2 Register 0x%x\n",
+                          L1SsCap.Capabilities, L1SsCap.Control1Reg, L1SsCap.Control1Reg);
+            }
+            else
+            {
+                NV_PRINTF(LEVEL_INFO,
+                          "L1 PM susbstates is not enabled in RootPort. L1 PM Control 1 Register 0x%x \n",
+                          L1SsCap.Control1Reg);
+            }
+        }
+        else
+        {
+            NV_PRINTF(LEVEL_INFO, "Failed to get L1 PM substates capabilites from Root Port\n");
+        }
+    }
+}
+
 /*!
  * @brief Initialize DMA caps
  *
@@ -334,7 +421,10 @@ kbifStaticInfoInit_IMPL
                             pStaticInfo->bPcieGen4Capable);
     pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_IS_C2C_LINK_UP,
                             pStaticInfo->bIsC2CLinkUp);
-    pKernelBif->dmaWindowStartAddress = pStaticInfo->dmaWindowStartAddress;
+    pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_DEVICE_IS_MULTIFUNCTION,
+                            pStaticInfo->bIsDeviceMultiFunction);
+    pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_GCX_PMU_CFG_SPACE_RESTORE,
+                            pStaticInfo->bGcxPmuCfgSpaceRestore);
 
 kBifStaticInfoInit_IMPL_exit:
     portMemFree(pStaticInfo);
@@ -346,7 +436,7 @@ kBifStaticInfoInit_IMPL_exit:
  * @brief Initialize PCI-E config space bits based on chipset and GPU support.
  */
 void
-kbifInitPcieDeviceControlStatus
+kbifInitPcieDeviceControlStatus_IMPL
 (
     OBJGPU    *pGpu,
     KernelBif *pKernelBif
@@ -372,7 +462,7 @@ kbifInitPcieDeviceControlStatus
         kbifPcieConfigDisableRelaxedOrdering_HAL(pGpu, pKernelBif);
     }
 
-    // 
+    //
     // WAR for bug 3661529. All GH100 SKUs will need the NoSnoop WAR.
     // But currently GSP-RM does not detect this correctly,
     //
@@ -417,7 +507,8 @@ kbifCheckAndRearmMSI_IMPL
 
     if (kbifIsMSIEnabled(pGpu, pKernelBif))
     {
-        if (!IS_VIRTUAL(pGpu))
+        if (!IS_VIRTUAL(pGpu) ||
+            pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_ALLOW_REARM_MSI_FOR_VF))
         {
             // Send EOI to rearm
             if (pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_USE_CONFIG_SPACE_TO_REARM_MSI))
@@ -515,7 +606,7 @@ kbifIsMSIXEnabled_IMPL
 }
 
 /*!
- * @brief Clear PCIe HW PCIe config space error counters. 
+ * @brief Clear PCIe HW PCIe config space error counters.
  * All of these should be cleared using config cycles.
  *
  * @param[in]   pGpu          GPU object pointer
@@ -704,6 +795,78 @@ _kbifInitRegistryOverrides
         NV_PRINTF(LEVEL_INFO, "allow peermapping reg key = %d\n", data32);
         pKernelBif->peerMappingOverride = !!data32;
     }
+
+    // Check if the PCIe SBR recovery feature is enabled.
+    if (osReadRegistryDword(pGpu, NV_REG_STR_SECONDARY_BUS_RESET_ENABLED, &data32) == NV_OK)
+    {
+        if (data32)
+        {
+            // The regkey was set to a non-zero value, enable the feature.
+            pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_SECONDARY_BUS_RESET_ENABLED, NV_TRUE);
+        }
+        else
+        {
+            // The regkey was set to 0, disable the feature.
+            pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_SECONDARY_BUS_RESET_ENABLED, NV_FALSE);
+        }
+    }
+
+    // Check if saving of the PCIe config space during SBR is enabled.
+    if (osReadRegistryDword(pGpu, NV_REG_STR_FORCE_PCIE_CONFIG_SAVE, &data32) == NV_OK)
+    {
+        if (data32)
+        {
+            // The regkey was set to a non-zero value, enable config save
+            pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_FORCE_PCIE_CONFIG_SAVE, NV_TRUE);
+        }
+        else
+        {
+            // The regkey was set to a 0, disable config save
+            pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_FORCE_PCIE_CONFIG_SAVE, NV_FALSE);
+        }
+    }
+
+    // Check if Function Level Reset(FLR) is disabled
+    pKernelBif->bForceDisableFLR = NV_REG_STR_RM_PCIE_FLR_POLICY_DEFAULT;
+    if (osReadRegistryDword(pGpu,
+           NV_REG_STR_RM_PCIE_FLR_POLICY, &data32) == NV_OK)
+    {
+        NV_PRINTF(LEVEL_INFO, "Pcie FLR Policy reg key = %d\n", data32);
+
+        if (data32 == NV_REG_STR_RM_PCIE_FLR_POLICY_FORCE_DISABLE)
+        {
+            pKernelBif->bForceDisableFLR = NV_TRUE;
+        }
+        // There might be a requirement to handle (data32 == NV_REG_STR_RM_PCIE_FLR_POLICY_DEFAULT) case as per platform's policy
+        else
+        {
+            pKernelBif->bForceDisableFLR = NV_FALSE;
+        }
+    }
+
+    // Check for FLR timeout scale override
+    if (osReadRegistryDword(pGpu,
+           NV_REG_STR_RM_PCIE_FLR_DEVINIT_TIMEOUT_SCALE, &data32) == NV_OK)
+    {
+        if ((data32 >= NV_REG_STR_RM_PCIE_FLR_DEVINIT_TIMEOUT_SCALE_MIN_ALLOWED) &&
+            (data32 <= NV_REG_STR_RM_PCIE_FLR_DEVINIT_TIMEOUT_SCALE_MAX_ALLOWED))
+        {
+            pKernelBif->flrDevInitTimeoutScale = data32;
+        }
+    }
+
+    // RmWar5045021 added for bug5045021
+    if (osReadRegistryDword(pGpu, NV_REG_STR_RM_WAR_5045021, &data32) == NV_OK)
+    {
+        if (data32)
+        {
+            pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_WAR_5045021_ENABLED, NV_TRUE);
+        }
+        else
+        {
+            pKernelBif->setProperty(pKernelBif, PDB_PROP_KBIF_WAR_5045021_ENABLED, NV_FALSE);
+        }
+    }
 }
 
 /*!
@@ -723,13 +886,21 @@ _kbifCheckIfGpuExists
     {
         if (gpuVerifyExistence_HAL(pGpu) != NV_OK)
         {
-            osRemove1SecondRepeatingCallback(pGpu, _kbifCheckIfGpuExists, NULL);
+            osRemove1HzCallback(pGpu, _kbifCheckIfGpuExists, NULL);
         }
     }
 }
 
-static NvU32
-kbifGetGpuLinkCapabilities
+/*!
+ * @brief Get link capabilities register value
+ *
+ * @param[in] pGpu       GPU object pointer
+ * @param[in] pKernelBif KernelBif object pointer
+ *
+ * @return    Register value
+ */
+NvU32
+kbifGetGpuLinkCapabilities_IMPL
 (
     OBJGPU    *pGpu,
     KernelBif *pKernelBif
@@ -746,14 +917,25 @@ kbifGetGpuLinkCapabilities
     if (NV_OK != GPU_BUS_CFG_RD32(pGpu, addrLinkCap, &data))
     {
         NV_PRINTF(LEVEL_ERROR, "Unable to read %x\n", addrLinkCap);
-        return 0;
+        data = 0;
     }
 
+    // Cache link capabilities here to be used by GSP-RM
+    pKernelBif->pcieConfigReg.linkCap = data;
+
     return data;
 }
 
-static NvU32
-kbifGetGpuLinkControlStatus
+/*!
+ * @brief Get link control status register value
+ *
+ * @param[in] pGpu       GPU object pointer
+ * @param[in] pKernelBif KernelBif object pointer
+ *
+ * @return    Register value
+ */
+NvU32
+kbifGetGpuLinkControlStatus_IMPL
 (
     OBJGPU    *pGpu,
     KernelBif *pKernelBif
@@ -769,7 +951,103 @@ kbifGetGpuLinkControlStatus
 
     if (NV_OK != GPU_BUS_CFG_RD32(pGpu, addrLinkControlStatus, &data ))
     {
-        NV_PRINTF(LEVEL_ERROR, "Unable to read %x\n", addrLinkControlStatus);
+        NV_PRINTF(LEVEL_INFO, "Unable to read %x\n", addrLinkControlStatus);
+        return 0;
+    }
+
+    return data;
+}
+
+/*!
+ * @brief Get device control status register value
+ *
+ * @param[in] pGpu       GPU object pointer
+ * @param[in] pKernelBif KernelBif object pointer
+ *
+ * @return    Register value
+ */
+NvU32
+kbifGetGpuDevControlStatus_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    NvU32 addrDevControlStatus = 0;
+    NvU32 data                 = 0;
+
+    if (NV_OK != kbifGetBusOptionsAddr_HAL(pGpu, pKernelBif, BUS_OPTIONS_DEV_CONTROL_STATUS, &addrDevControlStatus))
+    {
+        return 0;
+    }
+
+    if (NV_OK != GPU_BUS_CFG_RD32(pGpu, addrDevControlStatus, &data ))
+    {
+        NV_PRINTF(LEVEL_INFO, "Unable to read %x\n", addrDevControlStatus);
+        return 0;
+    }
+
+    return data;
+}
+
+/*!
+ * @brief Get device control status 2 register value
+ *
+ * @param[in] pGpu       GPU object pointer
+ * @param[in] pKernelBif KernelBif object pointer
+ *
+ * @return    Register value
+ */
+NvU32
+kbifGetGpuDevControlStatus2_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    NvU32 addrDevControlStatus2 = 0;
+    NvU32 data                  = 0;
+
+    if (NV_OK != kbifGetBusOptionsAddr_HAL(pGpu, pKernelBif, BUS_OPTIONS_DEV_CONTROL_STATUS_2, &addrDevControlStatus2))
+    {
+        return 0;
+    }
+
+    if (NV_OK != GPU_BUS_CFG_RD32(pGpu, addrDevControlStatus2, &data ))
+    {
+        NV_PRINTF(LEVEL_INFO, "Unable to read %x\n", addrDevControlStatus2);
+        return 0;
+    }
+
+    return data;
+}
+
+/*!
+ * @brief Get L1 Substates control register value
+ *
+ * @param[in] pGpu       GPU object pointer
+ * @param[in] pKernelBif KernelBif object pointer
+ *
+ * @return    Register value
+ */
+NvU32
+kbifGetGpuL1PmSubstatesCtrl1_IMPL
+(
+    OBJGPU    *pGpu,
+    KernelBif *pKernelBif
+)
+{
+    NvU32 addrL1PmSubstatesCtrl1 = 0;
+    NvU32 data                   = 0;
+
+    if (NV_OK != kbifGetBusOptionsAddr_HAL(pGpu, pKernelBif, BUS_OPTIONS_L1_PM_SUBSTATES_CTRL_1, &addrL1PmSubstatesCtrl1))
+    {
+        return 0;
+    }
+
+    if (NV_OK != GPU_BUS_CFG_RD32(pGpu, addrL1PmSubstatesCtrl1, &data ))
+    {
+        NV_PRINTF(LEVEL_INFO, "Unable to read %x\n", addrL1PmSubstatesCtrl1);
         return 0;
     }
 
@@ -806,10 +1084,11 @@ kbifControlGetPCIEInfo_IMPL
     NvU32   index = pBusInfo->index;
     NvU32   data  = 0;
 
-    if (kbifGetBusIntfType_HAL(pKernelBif) != NV2080_CTRL_BUS_INFO_TYPE_PCI_EXPRESS)
+    if ((pKernelBif != NULL) &&
+        (kbifGetBusIntfType_HAL(pKernelBif) != NV2080_CTRL_BUS_INFO_TYPE_PCI_EXPRESS))
     {
-        // KMD cannot handle error codes for this ctrl call, hence returning 
-        // NV_OK, once KMD fixes the bug:3545197, RM can return NV_ERR_NOT_SUPPORTED 
+        // KMD cannot handle error codes for this ctrl call, hence returning
+        // NV_OK, once KMD fixes the bug:3545197, RM can return NV_ERR_NOT_SUPPORTED
         return NV_OK;
     }
 
@@ -935,7 +1214,7 @@ kbifControlGetPCIEInfo_IMPL
                 {
                     NV2080_CTRL_BUS_INFO busInfo = {0};
                     NV_STATUS rmStatus = NV_OK;
- 
+
                     busInfo.index = NV2080_CTRL_BUS_INFO_INDEX_PCIE_GEN_INFO;
 
                     if ((rmStatus = kbusSendBusInfo(pGpu, GPU_GET_KERNEL_BUS(pGpu), &busInfo)) != NV_OK)
@@ -1130,7 +1409,6 @@ kbifPollDeviceOnBus_IMPL
     KernelBif  *pKernelBif
 )
 {
-    NV_STATUS status = NV_OK;
     RMTIMEOUT timeout;
 
     gpuSetTimeout(pGpu, GPU_TIMEOUT_DEFAULT, &timeout, 0);
@@ -1141,13 +1419,167 @@ kbifPollDeviceOnBus_IMPL
     {
         if (gpuCheckTimeout(pGpu, &timeout) == NV_ERR_TIMEOUT)
         {
-            status = NV_ERR_TIMEOUT;
             NV_PRINTF(LEVEL_ERROR, "Timeout polling GPU back on bus\n");
             DBG_BREAKPOINT();
+            return NV_ERR_TIMEOUT;
         }
         osDelayUs(100);
     }
 
+    return NV_OK;
+}
+
+/*!
+ * @brief To get PCI link max spead from PCI link Gen info
+ *
+ * @param[in]  pGpu        GPU object pointer
+ * @param[in]  pKernelBif  Kernel BIF object pointer
+ * @param[in]  pciLinkGenInfo   PCI link Gen info
+ * @param[in]  pciLinkMaxSpeed  pointer to PCI link max spead
+
+ * @returns NV_OK
+ * @returns NV_ERR_INVALID_STATE
+ */
+NV_STATUS
+kbifGetPciLinkMaxSpeedByPciGenInfo_IMPL
+(
+    OBJGPU     *pGpu,
+    KernelBif  *pKernelBif,
+    NvU32      pciLinkGenInfo,
+    NvU32      *pciLinkMaxSpeed
+)
+{
+    NV_STATUS rmStatus = NV_OK;
+
+    switch (pciLinkGenInfo)
+    {
+        case NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_GEN_GEN1:
+            *pciLinkMaxSpeed = NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_MAX_SPEED_2500MBPS;
+            break;
+
+        case NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_GEN_GEN2:
+            *pciLinkMaxSpeed = NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_MAX_SPEED_5000MBPS;
+            break;
+
+        case NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_GEN_GEN3:
+            *pciLinkMaxSpeed = NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_MAX_SPEED_8000MBPS;
+            break;
+
+        case NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_GEN_GEN4:
+            *pciLinkMaxSpeed = NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_MAX_SPEED_16000MBPS;
+            break;
+
+        case NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_GEN_GEN5:
+            *pciLinkMaxSpeed = NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_MAX_SPEED_32000MBPS;
+            break;
+
+        case NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_GEN_GEN6:
+            *pciLinkMaxSpeed = NV2080_CTRL_BUS_INFO_PCIE_LINK_CAP_MAX_SPEED_64000MBPS;
+            break;
+
+        default:
+            rmStatus = NV_ERR_INVALID_STATE;
+            NV_PRINTF(LEVEL_ERROR, "Unknown PCIe Gen Info\n");
+    }
+
+    return rmStatus;
+}
+
+NV_STATUS
+kbifResetFromTimeoutFullChip_IMPL
+(
+    OBJGPU *pGpu,
+    KernelBif *pKernelBif
+)
+{
+
+    OBJSYS *pSys = SYS_GET_INSTANCE();
+    OBJCL  *pCl = SYS_GET_CL(pSys);
+    NV_STATUS status;
+
+    if (pCl == NULL || 
+        (!pCl->getProperty(pCl, PDB_PROP_CL_PCIE_CONFIG_ACCESSIBLE)
+       ))
+    {
+        //
+        // We can not issue SW_RESET as we are not able to access PCI config
+        // space.
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    //
+    // Execute any workarounds or changes needed to make sure we can reset
+    // properly.
+    //
+    kbifPrepareForFullChipReset_HAL(pGpu, pKernelBif);
+
+    // Reset the gpu.
+    NV_ASSERT_OK(status = kbifDoFullChipReset_HAL(pGpu, pKernelBif));
+
+    //
+    // This 100ms wait is only required for legacy FLR where RM itself
+    // triggers FLR and it also does save/restore of config space.
+    // For hopper and later, HW will keep sending CRS status to OS
+    // until FWSEC boot sequence is not complete
+    //
+    if (!(pKernelBif->getProperty(pKernelBif, PDB_PROP_KBIF_FLR_HANDLED_BY_OS)))
+    {
+        // Wait 100ms after GPU is reset to ensure Pre-OS completion.
+        osDelay(100);
+    }
+
     return status;
 }
 
+NV_STATUS
+kbifWaitForConfigAccessAfterReset_IMPL
+(
+    OBJGPU *pGpu,
+    KernelBif *pKernelBif
+)
+{
+
+
+    NvU32 domain = gpuGetDomain(pGpu);
+    NvU8  bus    = gpuGetBus(pGpu);
+    NvU8  device = gpuGetDevice(pGpu);
+
+    NvU32 waitCount = 20; // iterations of the osDelayUs statements
+    while (NV_TRUE)
+    {
+        NvU16 vendorId;
+        NvU16 deviceId;
+        void *pGpuHandle = osPciInitHandle(domain,
+                                           bus,
+                                           device,
+                                           0 /* Function */,
+                                           &vendorId,
+                                           &deviceId);
+
+        NvU32 id = osPciReadDword(pGpuHandle, 0 /* Offset */);
+        if (id == pGpu->idInfo.PCIDeviceID)
+        {
+            // Found it.  Config space is accessible.  Break out.
+            return NV_OK;
+        }
+
+        if (waitCount == 0)
+        {
+            break;
+        }
+        waitCount--;
+
+        //
+        // Keep trying for at least 1 second.  Note that completion timeouts
+        // here can extend the amount of time we are in this loop.  But they are
+        // not fatal.
+        //
+        {
+            // Wait 50ms
+            osDelayUs(50000);
+        }
+    }
+
+    return NV_ERR_GENERIC;
+}
+
