@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2000-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2000-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -31,6 +31,7 @@
 
 #include "platform/platform.h"
 #include "platform/chipset/chipset.h"
+#include "gpu/bif/kernel_bif.h"
 #include "gpu/mem_mgr/mem_mgr.h"
 #include "gpu/mem_mgr/fbsr.h"
 #include "gpu/gsp/gsp_init_args.h"
@@ -40,12 +41,14 @@
 #include "core/thread_state.h"
 #include "vgpu/rpc.h"
 #include "gpu/mem_sys/kern_mem_sys.h"
+#include <gpu/fsp/kern_fsp.h>
 
 //
 // Helper functions
 //
 static NV_STATUS gpuPowerManagementEnter(OBJGPU *, NvU32 newLevel, NvU32 flags);
 static NV_STATUS gpuPowerManagementResume(OBJGPU *, NvU32 oldLevel, NvU32 flags);
+static NV_STATUS _gpuPollCFGAndCheckD3Hot(OBJGPU *);
 
 // XXX Needs to be further cleaned up. No new code should be placed in this
 // routine. Please use the per-engine StateLoad() and StateUnload() routines
@@ -56,6 +59,32 @@ gpuPowerManagementEnter(OBJGPU *pGpu, NvU32 newLevel, NvU32 flags)
     NV_STATUS  status = NV_OK;
     MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
 
+    pGpu->powerManagementDepth = NV_PM_DEPTH_OS_LAYER;
+
+    KernelGsp *pKernelGsp = GPU_GET_KERNEL_GSP(pGpu);
+    KernelGspUnloadMode unloadMode = KGSP_UNLOAD_MODE_SR_SUSPEND;
+
+    if (IS_GSP_CLIENT(pGpu))
+    {
+        if (IS_GPU_GC6_STATE_ENTERING(pGpu))
+        {
+            unloadMode = KGSP_UNLOAD_MODE_GC6_ENTER;
+        }
+        else
+        {
+            unloadMode = KGSP_UNLOAD_MODE_SR_SUSPEND;
+
+            // Prepare SR metadata structure for suspend
+            status = kgspPrepareSuspendResumeData_HAL(pGpu, pKernelGsp);
+            if (status != NV_OK)
+            {
+                return status;
+            }
+        }
+
+        pGpu->powerManagementDepth = NV_PM_DEPTH_SR_META;
+    }
+
     // This is a no-op in CPU-RM
     NV_ASSERT_OK_OR_GOTO(status, gpuPowerManagementEnterPreUnloadPhysical(pGpu), done);
 
@@ -64,6 +93,7 @@ gpuPowerManagementEnter(OBJGPU *pGpu, NvU32 newLevel, NvU32 flags)
         GPU_STATE_FLAGS_PRESERVING | GPU_STATE_FLAGS_PM_TRANSITION | GPU_STATE_FLAGS_GC6_TRANSITION :
         GPU_STATE_FLAGS_PRESERVING | GPU_STATE_FLAGS_PM_TRANSITION), done);
 
+    pGpu->powerManagementDepth = NV_PM_DEPTH_STATE_LOAD;
     pGpu->setProperty(pGpu, PDB_PROP_GPU_VGA_ENABLED, NV_TRUE);
 
     // This is a no-op in CPU-RM
@@ -71,46 +101,22 @@ gpuPowerManagementEnter(OBJGPU *pGpu, NvU32 newLevel, NvU32 flags)
 
     if (IS_GSP_CLIENT(pGpu))
     {
-        NV_ASSERT_OK_OR_GOTO(status, memmgrSavePowerMgmtState(pGpu, pMemoryManager), done);
-
-        KernelGsp *pKernelGsp = GPU_GET_KERNEL_GSP(pGpu); 
-
-
-        NV_RM_RPC_UNLOADING_GUEST_DRIVER(pGpu, status, NV_TRUE, IS_GPU_GC6_STATE_ENTERING(pGpu), newLevel);
+        status = kgspUnloadRm(pGpu, pKernelGsp, unloadMode, newLevel);
         if (status != NV_OK)
+        {
+            NV_PRINTF(LEVEL_ERROR, "GSP unload failed at suspend (bootMode 0x%x, newLevel 0x%x): 0x%x\n",
+                      unloadMode, newLevel, status);
             goto done;
-
-        // Wait for GSP-RM to suspend
-        kgspWaitForProcessorSuspend_HAL(pGpu, pKernelGsp);
-
-        // Dump GSP-RM logs before resetting and invoking FWSEC-SB
-        kgspDumpGspLogs(pGpu, pKernelGsp, NV_FALSE);
-
-        // Because of COT, RM cannot reset GSP-RISCV.
-        if (!(pGpu->getProperty(pGpu, PDB_PROP_GPU_IS_COT_ENABLED)))
-        {
-            kflcnReset_HAL(pGpu, staticCast(pKernelGsp, KernelFalcon));
         }
 
-        // Invoke FWSEC-SB to load back PreOsApps.
-        if (kgspExecuteFwsecSb_HAL(pGpu, pKernelGsp, pKernelGsp->pFwsecUcode) != NV_OK)
+        if (!IS_GPU_GC6_STATE_ENTERING(pGpu))
         {
-            NV_PRINTF(LEVEL_ERROR, "failed to execute FWSEC-SB for PreOsApps\n");
-        } 
-
-        kpmuFreeLibosLoggingStructures(pGpu, GPU_GET_KERNEL_PMU(pGpu));
-
-        {
-            // Not called when kgspShouldBootWithBooter_HAL() is called and returns NV_FALSE
-            NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-                                kgspSavePowerMgmtState_HAL(pGpu, pKernelGsp), done);
-
+            kpmuFreeLibosLoggingStructures(pGpu, GPU_GET_KERNEL_PMU(pGpu));
         }
-
     }
 
 done:
-    if (status != NV_OK)
+    if ((status != NV_OK) && !IS_GPU_GC6_STATE_ENTERING(pGpu))
     {
         memmgrFreeFbsrMemory(pGpu, pMemoryManager);
     }
@@ -133,6 +139,11 @@ gpuPowerManagementResume(OBJGPU *pGpu, NvU32 oldLevel, NvU32 flags)
     OBJCL          *pCl    = SYS_GET_CL(pSys);
     MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
 
+    if (pGpu->powerManagementDepth == NV_PM_DEPTH_OS_LAYER)
+    {
+        return NV_OK;
+    }
+
 #ifdef DEBUG
     //
     // This is useful for windbg debugging as it frequently doesn't reconnect to the
@@ -161,7 +172,8 @@ gpuPowerManagementResume(OBJGPU *pGpu, NvU32 oldLevel, NvU32 flags)
         //
         kmemsysProgramSysmemFlushBuffer_HAL(pGpu, GPU_GET_KERNEL_MEMORY_SYSTEM(pGpu));
 
-        KernelGsp *pKernelGsp = GPU_GET_KERNEL_GSP(pGpu); 
+        KernelGsp *pKernelGsp = GPU_GET_KERNEL_GSP(pGpu);
+        KernelGspBootMode bootMode;
 
         GSP_SR_INIT_ARGUMENTS gspSrInitArgs;
 
@@ -178,26 +190,56 @@ gpuPowerManagementResume(OBJGPU *pGpu, NvU32 oldLevel, NvU32 flags)
             goto done;
         }
 
-        // Not called when kgspShouldBootWithBooter_HAL() is called and returns NV_FALSE
+        if (IS_GPU_GC6_STATE_EXITING(pGpu))
         {
-            NV_CHECK_OK_OR_GOTO(status, LEVEL_ERROR,
-                                kgspRestorePowerMgmtState_HAL(pGpu, pKernelGsp), done);
+            bootMode = KGSP_BOOT_MODE_GC6_EXIT;
+        }
+        else
+        {
+            bootMode = KGSP_BOOT_MODE_SR_RESUME;
+
+            status = kpmuInitLibosLoggingStructures(pGpu, GPU_GET_KERNEL_PMU(pGpu));
+            if (status != NV_OK)
+            {
+                NV_PRINTF(LEVEL_ERROR, "cannot init libOS PMU logging structures: 0x%x\n", status);
+                goto done;
+            }
         }
 
-        status = kpmuInitLibosLoggingStructures(pGpu, GPU_GET_KERNEL_PMU(pGpu));
+        status = kgspPrepareForBootstrap_HAL(pGpu, pKernelGsp, bootMode);
         if (status != NV_OK)
         {
-            NV_PRINTF(LEVEL_ERROR, "cannot init libOS PMU logging structures: 0x%x\n", status);
+            NV_PRINTF(LEVEL_ERROR, "GSP boot preparation failed at resume (bootMode 0x%x): 0x%x\n", bootMode, status);
             goto done;
         }
 
-        status = kgspWaitForRmInitDone(pGpu, pKernelGsp);
+        status = kgspBootstrap_HAL(pGpu, pKernelGsp, bootMode);
         if (status != NV_OK)
         {
-            NV_PRINTF(LEVEL_ERROR, "State load at resume for riscv/gsp failed: 0x%x\n", status);
+            NV_PRINTF(LEVEL_ERROR, "GSP boot failed at resume (bootMode 0x%x): 0x%x\n", bootMode, status);
             goto done;
         }
     }
+    else
+    {
+        // Boot GSP-FMC for monolithic RM
+        KernelFsp *pKernelFsp = GPU_GET_KERNEL_FSP(pGpu);
+        if ((pKernelFsp != NULL) && !IS_VIRTUAL(pGpu))
+        {
+            pKernelFsp->setProperty(pKernelFsp, PDB_PROP_KFSP_BOOT_COMMAND_OK, NV_FALSE);
+
+            // This is a no-op in CPU-RM
+            NV_ASSERT_OK_OR_GOTO(status, kfspPrepareKeepWPRAcrossGc6Physical(pGpu, pKernelFsp), done);
+
+            status = kfspPrepareAndSendBootCommands_HAL(pGpu, pKernelFsp);
+            if (status != NV_OK)
+            {
+                NV_PRINTF(LEVEL_ERROR, "FSP boot command failed during resume.\n");
+                goto done;
+            }
+        }
+
+    }
 
     // This is a no-op in CPU-RM
     NV_ASSERT_OK_OR_GOTO(status, gpuPowerManagementResumePreLoadPhysical(pGpu, oldLevel, flags), done);
@@ -215,11 +257,61 @@ gpuPowerManagementResume(OBJGPU *pGpu, NvU32 oldLevel, NvU32 flags)
     NV_PRINTF(LEVEL_NOTICE, "Adapter now in D0 state\n");
 
 done:
-    memmgrFreeFbsrMemory(pGpu, pMemoryManager);
+    if (!IS_GPU_GC6_STATE_EXITING(pGpu))
+    {
+        if (IS_GSP_CLIENT(pGpu))
+        {
+            KernelGsp *pKernelGsp = GPU_GET_KERNEL_GSP(pGpu);
+            kgspFreeSuspendResumeData_HAL(pGpu, pKernelGsp);
+        }
+
+        memmgrFreeFbsrMemory(pGpu, pMemoryManager);
+    }
 
     return status;
 }
 
+//
+// Since the BAR firewall introduced from Blackwell
+// Need to poll Device on the bus and CFG BAR firewall first,
+// before any PRI register accessing.
+// For GCOFF/GC8/GC6 three cases, all will run into standby path,
+// So adding this helper function before calling gpu resume.
+//
+static NV_STATUS
+_gpuPollCFGAndCheckD3Hot(OBJGPU *pGpu)
+{
+    KernelBif *pKernelBif = GPU_GET_KERNEL_BIF(pGpu);
+
+    //
+    // GCOFF/GC8/GC6 three cases, all will run into here.
+    // Because of BAR firewall introduced from Blackwell,
+    // We need to poll CFG for:
+    // 1. if GPU is on the bus
+    // 2. if BAR firewall disengage only if blackwell and later
+    // 3. check GC6 D3Hot case (has PRI register access, must after firewall disengage)
+    //
+
+    if (!IS_FMODEL(pGpu))
+    {
+        // Polling GPU on the bus
+        NV_ASSERT_OK_OR_RETURN(kbifPollDeviceOnBus(pGpu, pKernelBif));
+
+        //
+        // For V0/V0S fused board, BAR firewall introduced from Blackwell onwards.
+        // Polling BAR firewall disengage if FSP released before accessing any PRI access.
+        //
+        NV_ASSERT_OK_OR_RETURN(kbifPollBarFirewallDisengage_HAL(pGpu, pKernelBif));
+    }
+
+    // For GC6 exit, check if D3Hot case
+    if(IS_GPU_GC6_STATE_EXITING(pGpu))
+    {
+        gpuCheckGc6inD3Hot(pGpu);
+    }
+    return NV_OK;
+}
+
 NV_STATUS
 gpuEnterStandby_IMPL(OBJGPU *pGpu)
 {
@@ -244,6 +336,8 @@ gpuEnterStandby_IMPL(OBJGPU *pGpu)
 
     pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_PM_CODEPATH, NV_TRUE);
 
+    gpuNotifySubDeviceEvent(pGpu, NV2080_NOTIFIERS_POWER_SUSPEND, NULL, 0, 0, 0);
+
     suspendStatus = gpuPowerManagementEnter(pGpu, NV2080_CTRL_GPU_SET_POWER_STATE_GPU_LEVEL_3, GPU_STATE_FLAGS_PM_SUSPEND);
 
     pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_STANDBY, NV_TRUE);
@@ -290,6 +384,12 @@ gpuResumeFromStandby_IMPL(OBJGPU *pGpu)
 
     pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_PM_RESUME_CODEPATH, NV_TRUE);
 
+    resumeStatus = _gpuPollCFGAndCheckD3Hot(pGpu);
+    if (resumeStatus != NV_OK)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Polling BAR0 or BAR firewall timeout\n");
+    }
+
     resumeStatus = gpuPowerManagementResume(pGpu, state, GPU_STATE_FLAGS_PM_SUSPEND);
 
     pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_STANDBY, NV_FALSE);
@@ -297,6 +397,9 @@ gpuResumeFromStandby_IMPL(OBJGPU *pGpu)
     pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_PM_CODEPATH, NV_FALSE);
     pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_PM_RESUME_CODEPATH, NV_FALSE);
 
+    // reset PM depth on resume
+    pGpu->powerManagementDepth = NV_PM_DEPTH_NONE;
+
     if ((pPfm != NULL) && pPfm->getProperty(pPfm, PDB_PROP_PFM_SUPPORTS_ACPI))
     {
         NV_PRINTF(LEVEL_NOTICE, "Ending transition from %s to D0\n",
@@ -308,6 +411,8 @@ gpuResumeFromStandby_IMPL(OBJGPU *pGpu)
                   IS_GPU_GC6_STATE_EXITING(pGpu) ? "GC6" : "APM Suspend");
     }
 
+    gpuNotifySubDeviceEvent(pGpu, NV2080_NOTIFIERS_POWER_RESUME, NULL, 0, 0, 0);
+
     return resumeStatus;
 }
 
@@ -332,6 +437,8 @@ NV_STATUS gpuEnterHibernate_IMPL(OBJGPU *pGpu)
 
     pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_PM_CODEPATH, NV_TRUE);
 
+    gpuNotifySubDeviceEvent(pGpu, NV2080_NOTIFIERS_POWER_SUSPEND, NULL, 0, 0, 0);
+
     suspendStatus = gpuPowerManagementEnter(pGpu, NV2080_CTRL_GPU_SET_POWER_STATE_GPU_LEVEL_7, GPU_STATE_FLAGS_PM_HIBERNATE);
 
     NV_PRINTF(LEVEL_INFO,
@@ -362,6 +469,9 @@ NV_STATUS gpuResumeFromHibernate_IMPL(OBJGPU *pGpu)
     pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_PM_CODEPATH, NV_FALSE);
     pGpu->setProperty(pGpu, PDB_PROP_GPU_IN_PM_RESUME_CODEPATH, NV_FALSE);
 
+    // reset PM depth on resume
+    pGpu->powerManagementDepth = NV_PM_DEPTH_NONE;
+
     if ((pPfm != NULL) && pPfm->getProperty(pPfm, PDB_PROP_PFM_SUPPORTS_ACPI))
     {
         NV_PRINTF(LEVEL_NOTICE, "Ending transition from D4 to D0\n");
@@ -371,5 +481,7 @@ NV_STATUS gpuResumeFromHibernate_IMPL(OBJGPU *pGpu)
         NV_PRINTF(LEVEL_NOTICE, "End resuming from APM Suspend\n");
     }
 
+    gpuNotifySubDeviceEvent(pGpu, NV2080_NOTIFIERS_POWER_RESUME, NULL, 0, 0, 0);
+
     return resumeStatus;
 }
