@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -30,6 +30,8 @@
 *                                                                          *
 \***************************************************************************/
 
+#define NVOC_KERNEL_SM_DEBUGGER_SESSION_H_PRIVATE_ACCESS_ALLOWED
+
 #include "kernel/rmapi/control.h"
 #include "kernel/rmapi/rmapi.h"
 #include "kernel/os/os.h"
@@ -49,6 +51,7 @@
 
 #include "class/cl83de.h"
 #include "class/clc637.h"
+#include "class/cl0071.h" // NV01_MEMORY_SYSTEM_OS_DESCRIPTOR
 #include "ctrl/ctrl83de.h"
 
 //
@@ -58,14 +61,14 @@
 // for the caller to explicitly pass in the handle corresponding to the VaSpaceApi:
 //
 static NV_STATUS
-_nv83deCtrlCmdFetchVAS(NvU32 hClient, NvU32 hChannel, OBJVASPACE **ppVASpace)
+_nv83deCtrlCmdFetchVAS(RsClient *pClient, NvU32 hChannel, OBJVASPACE **ppVASpace)
 {
     KernelChannel *pKernelChannel = NULL;
 
     NV_ASSERT_OR_RETURN(ppVASpace != NULL, NV_ERR_INVALID_ARGUMENT);
 
     // Fetch the corresponding Channel object from our handle
-    NV_ASSERT_OK_OR_RETURN(CliGetKernelChannel(hClient, hChannel, &pKernelChannel));
+    NV_ASSERT_OK_OR_RETURN(CliGetKernelChannel(pClient, hChannel, &pKernelChannel));
     NV_ASSERT_OR_RETURN(pKernelChannel != NULL, NV_ERR_INVALID_ARGUMENT);
 
     *ppVASpace = pKernelChannel->pVAS;
@@ -148,20 +151,20 @@ _nv8deCtrlCmdReadWriteSurface
 )
 {
     OBJGPU *pGpu = GPU_RES_GET_GPU(pKernelSMDebuggerSession);
-    NvHandle hClient = RES_GET_CLIENT_HANDLE(pKernelSMDebuggerSession);
+    RsClient *pClient = RES_GET_CLIENT(pKernelSMDebuggerSession);
     OBJVASPACE *pVASpace = NULL;
     NvU32 count = pParams->count;
     NvU32 i;
     NV_STATUS status = NV_OK;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
     if (count > MAX_ACCESS_OPS)
         return NV_ERR_INVALID_ARGUMENT;
 
     // Attempt to retrieve the VAS pointer
     NV_ASSERT_OK_OR_RETURN(
-        _nv83deCtrlCmdFetchVAS(hClient, pKernelSMDebuggerSession->hChannel, &pVASpace));
+        _nv83deCtrlCmdFetchVAS(pClient, pKernelSMDebuggerSession->hChannel, &pVASpace));
 
     // Validate VA range and fail if invalid
     NV_ASSERT_OK_OR_RETURN(
@@ -169,7 +172,6 @@ _nv8deCtrlCmdReadWriteSurface
 
     for (i = 0; i < count; i++)
     {
-        NvU8 *pBase;
         MEMORY_DESCRIPTOR *pMemDesc = NULL;
         NvU64 virtAddr = pParams->opsBuffer[i].gpuVA;
         NvP64 bufPtr = pParams->opsBuffer[i].pCpuVA;
@@ -187,12 +189,15 @@ _nv8deCtrlCmdReadWriteSurface
         end4kPage = (NvOffset_LO32(virtAddr + bufSize - 1) >> 12) & 0x1FFFF;
 
         curSize = RM_PAGE_SIZE - pageStartOffset;
-        virtAddr &= ~RM_PAGE_MASK;
+        virtAddr -= pageStartOffset;
 
         for (cur4kPage = start4kPage; cur4kPage <= end4kPage; ++cur4kPage)
         {
-            MMU_TRACE_PARAM mmuParams = {0};
-            MMU_TRACE_ARG traceArg    = {0};
+            MMU_TRACE_PARAM mmuParams     = {0};
+            MMU_TRACE_ARG traceArg        = {0};
+            TRANSFER_SURFACE surf         = {0};
+            MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+            NvU8 *pKernBuffer = NULL;
 
             mmuParams.mode    = MMU_TRACE_MODE_TRANSLATE;
             mmuParams.va      = virtAddr;
@@ -207,17 +212,23 @@ _nv8deCtrlCmdReadWriteSurface
                 curSize = bufSize;
             }
 
+            pKernBuffer = portMemAllocNonPaged(curSize);
+            if (pKernBuffer == NULL)
+            {
+                return NV_ERR_INSUFFICIENT_RESOURCES;
+            }
+
             if (traceArg.aperture == ADDR_SYSMEM)
             {
                 NvP64 physAddr = NV_PTR_TO_NvP64(traceArg.pa);
-                NvU64 limit = (NvU64)(curSize - 1);
+                NvU64 limit = RM_PAGE_SIZE - 1;
 
                 NvU32 os02Flags = DRF_DEF(OS02, _FLAGS, _LOCATION,      _PCI)           |
                                   DRF_DEF(OS02, _FLAGS, _MAPPING,       _NO_MAP)        |
                                   DRF_DEF(OS02, _FLAGS, _PHYSICALITY,   _CONTIGUOUS)    |
                                   DRF_DEF(OS02, _FLAGS, _COHERENCY,     _CACHED);
 
-                NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                NV_ASSERT_OK_OR_ELSE(status,
                     osCreateMemFromOsDescriptor(pGpu,
                                                 physAddr,
                                                 pKernelSMDebuggerSession->hInternalClient,
@@ -225,38 +236,55 @@ _nv8deCtrlCmdReadWriteSurface
                                                 &limit,
                                                 &pMemDesc,
                                                 NVOS32_DESCRIPTOR_TYPE_OS_PHYS_ADDR,
-                                                RS_PRIV_LEVEL_KERNEL));
+                                                RS_PRIV_LEVEL_KERNEL),
+                    portMemFree(pKernBuffer); return status; );
             }
             else if (traceArg.aperture == ADDR_FBMEM)
             {
-                memdescCreate(&pMemDesc, pGpu, curSize, 0, NV_TRUE, traceArg.aperture, NV_MEMORY_UNCACHED, MEMDESC_FLAGS_NONE);
-                memdescDescribe(pMemDesc, traceArg.aperture, traceArg.pa, curSize);
+                NV_ASSERT_OK_OR_ELSE(status,
+                    memdescCreate(&pMemDesc, pGpu, RM_PAGE_SIZE, 0, NV_TRUE,
+                                  traceArg.aperture, NV_MEMORY_UNCACHED,
+                                  MEMDESC_FLAGS_NONE),
+                    portMemFree(pKernBuffer); return status; );
+                memdescDescribe(pMemDesc, traceArg.aperture, traceArg.pa, RM_PAGE_SIZE);
             }
 
-            pBase = kbusMapRmAperture_HAL(pGpu, pMemDesc);
-            NV_ASSERT_OR_ELSE(
-                pBase != NULL,
-                memdescDestroy(pMemDesc);
-                return NV_ERR_INVALID_ARGUMENT; );
+            surf.pMemDesc = pMemDesc;
+            surf.offset = pageStartOffset;
 
             if (bWrite)
             {
                 NV_ASSERT_OK_OR_CAPTURE_FIRST_ERROR(status,
-                    portMemExCopyFromUser(bufPtr, pBase + pageStartOffset, curSize));
+                    portMemExCopyFromUser(bufPtr, pKernBuffer, curSize));
+
+                // Write out the buffer to memory
+                if (status == NV_OK)
+                {
+                    NV_ASSERT_OK_OR_CAPTURE_FIRST_ERROR(status,
+                        memmgrMemWrite(pMemoryManager, &surf, pKernBuffer, curSize,
+                                       TRANSFER_FLAGS_DEFER_FLUSH));
+                }
             }
             else
             {
+                // Read from memory
                 NV_ASSERT_OK_OR_CAPTURE_FIRST_ERROR(status,
-                    portMemExCopyToUser(pBase + pageStartOffset, bufPtr, curSize));
+                    memmgrMemRead(pMemoryManager, &surf, pKernBuffer, curSize,
+                                  TRANSFER_FLAGS_DEFER_FLUSH));
+
+                if (status == NV_OK)
+                {
+                    NV_ASSERT_OK_OR_CAPTURE_FIRST_ERROR(status,
+                        portMemExCopyToUser(pKernBuffer, bufPtr, curSize));
+                }
             }
 
-            kbusUnmapRmAperture_HAL(pGpu, pMemDesc, &pBase, NV_FALSE);
+            portMemFree(pKernBuffer);
             memdescDestroy(pMemDesc);
 
             if (status != NV_OK)
                 return status;
 
-            pBase = NULL;
             pageStartOffset = 0;
             bufPtr = NvP64_PLUS_OFFSET(bufPtr,curSize);
             bufSize -= curSize;
@@ -296,16 +324,16 @@ ksmdbgssnCtrlCmdGetMappings_IMPL
 )
 {
     OBJGPU *pGpu = GPU_RES_GET_GPU(pKernelSMDebuggerSession);
-    NvHandle hClient = RES_GET_CLIENT_HANDLE(pKernelSMDebuggerSession);
+    RsClient *pClient = RES_GET_CLIENT(pKernelSMDebuggerSession);
     OBJVASPACE *pVASpace = NULL;
     MMU_TRACE_ARG traceArg = {0};
     MMU_TRACE_PARAM mmuParams = {0};
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
     // Attempt to retrieve the VAS pointer
     NV_ASSERT_OK_OR_RETURN(
-        _nv83deCtrlCmdFetchVAS(hClient, pKernelSMDebuggerSession->hChannel, &pVASpace));
+        _nv83deCtrlCmdFetchVAS(pClient, pKernelSMDebuggerSession->hChannel, &pVASpace));
 
     traceArg.pMapParams = pParams;
 
@@ -478,9 +506,9 @@ _nv83deCtrlCmdDebugAccessMemory
 )
 {
     RsResourceRef *pResourceRef;
+    Memory *pMemory;
     MEMORY_DESCRIPTOR *pMemDesc;
     NvU64 totalLength;
-    NvP64 pCpuVirtAddr = NvP64_NULL;
     NV_STATUS rmStatus = NV_OK;
     NV_STATUS rmUnmapStatus = NV_OK;
     NvU32 flags = 0;
@@ -494,8 +522,8 @@ _nv83deCtrlCmdDebugAccessMemory
     if (serverutilGetResourceRef(hClient, hMemory, &pResourceRef) != NV_OK)
         return NV_ERR_INSUFFICIENT_PERMISSIONS;
 
-    // Get a memdesc for this object to determine its attributes
-    if (!dynamicCast(pResourceRef->pResource, Memory))
+    pMemory = dynamicCast(pResourceRef->pResource, Memory);
+    if (pMemory == NULL)
     {
         rmStatus = NV_ERR_INVALID_ARGUMENT;
         NV_PRINTF(LEVEL_WARNING,
@@ -510,7 +538,8 @@ _nv83deCtrlCmdDebugAccessMemory
         return rmStatus;
     }
 
-    pMemDesc = dynamicCast(pResourceRef->pResource, Memory)->pMemDesc;
+    // Get a memdesc for this object to determine its attributes
+    pMemDesc = pMemory->pMemDesc;
     if (pMemDesc == NULL)
         return NV_ERR_INVALID_STATE;
 
@@ -520,109 +549,132 @@ _nv83deCtrlCmdDebugAccessMemory
     if (totalLength > pMemDesc->Size)
         return NV_ERR_INVALID_ARGUMENT;
 
-    // Setup mapping flags based on the kind of memory, access type etc.
-    if (accessType == GRDBG_MEM_ACCESS_TYPE_READ)
-    {
-        flags = FLD_SET_DRF(OS33, _FLAGS, _ACCESS, _READ_ONLY, flags);
-    }
-    else
-    {
-        flags = FLD_SET_DRF(OS33, _FLAGS, _ACCESS, _WRITE_ONLY, flags);
-    }
-
     bCpuMemory = (memdescGetAddressSpace(pMemDesc) == ADDR_SYSMEM);
     bGpuCached = (memdescGetGpuCacheAttrib(pMemDesc) == NV_MEMORY_CACHED);
 
-    //
-    // Ask for a direct mapping to this memory, to avoid getting a reflected
-    // mapping. We'll do explicit cache management to ensure coherence.
-    //
-    if (bCpuMemory)
-    {
-        flags = FLD_SET_DRF(OS33, _FLAGS, _MAPPING, _DIRECT, flags);
-    }
-
-    // Map memory into the internal smdbg client
-    rmStatus = _nv83deMapMemoryIntoGrdbgClient(pTargetGpu,
-                                               pKernelSMDebuggerSession,
-                                               hClient,
-                                               hMemory,
-                                               offset,
-                                               length,
-                                               &pCpuVirtAddr,
-                                               flags);
-    if (NV_OK != rmStatus)
-    {
-        NV_PRINTF(LEVEL_WARNING,
-                  "Failed to map memory into internal smdbg client (GPU 0x%llx, hClient 0x%x, hMemory %x, offset 0x%llx, length 0x%x, flags 0x%x): (rmStatus = %x)\n",
-                  pTargetGpu->busInfo.gpuPhysAddr,
-                  hClient,
-                  hMemory,
-                  offset,
-                  length,
-                  flags,
-                  rmStatus);
-        return rmStatus;
-    }
-
-    // Fence to ensure previous in-flight accesses are complete
+    // Ensure previous in-flight accesses are complete
     osFlushCpuWriteCombineBuffer();
 
-    //
-    // Flush and invalidate SYSMEM lines from L2s of all GPUs.
-    // Some GPUs have write-back caches, so this must be done both for
-    // accessType == READ and accessType == WRITE.
-    //
     if (bCpuMemory && bGpuCached)
     {
-        rmStatus = _nv83deFlushAllGpusL2Cache(pMemDesc);
+        //
+        // Flush and invalidate SYSMEM lines from L2s of all GPUs.
+        // Some GPUs have write-back caches, so this must be done both for
+        // accessType == READ and accessType == WRITE.
+        //
+        NV_ASSERT_OK_OR_RETURN(_nv83deFlushAllGpusL2Cache(pMemDesc));
+    }
+
+    if (pMemory->categoryClassId == NV01_MEMORY_SYSTEM_OS_DESCRIPTOR)
+    {
+        // WAR: cudaHostRegister debugger test fails, temporary keep legacy behavior
+        return NV_ERR_NOT_SUPPORTED;
+    }
+
+    if (IS_VIRTUAL_WITHOUT_SRIOV(pTargetGpu) || IS_VIRTUAL_WITH_HEAVY_SRIOV(pTargetGpu)
+        || (!bCpuMemory && kbusIsBarAccessBlocked(GPU_GET_KERNEL_BUS(pTargetGpu)) &&
+        gpuIsCCDevToolsModeEnabled(pTargetGpu))
+    )
+    {
+        NvP64 pCpuVirtAddr = NvP64_NULL;
+
+        // Setup mapping flags based on the kind of memory, access type etc.
+        if (accessType == GRDBG_MEM_ACCESS_TYPE_READ)
+        {
+            flags = FLD_SET_DRF(OS33, _FLAGS, _ACCESS, _READ_ONLY, flags);
+        }
+        else
+        {
+            flags = FLD_SET_DRF(OS33, _FLAGS, _ACCESS, _WRITE_ONLY, flags);
+        }
+
+        if (!bCpuMemory &&
+            kbusIsBarAccessBlocked(GPU_GET_KERNEL_BUS(pTargetGpu)) &&
+            gpuIsCCDevToolsModeEnabled(pTargetGpu))
+        {
+            // Allow the mapping to happen successfully on HCC devtools mode
+            flags = FLD_SET_DRF(OS33, _FLAGS, _ALLOW_MAPPING_ON_HCC, _YES, flags);
+        }
+
+        // Map memory into the internal smdbg client
+        rmStatus = _nv83deMapMemoryIntoGrdbgClient(pTargetGpu,
+                                                   pKernelSMDebuggerSession,
+                                                   hClient,
+                                                   hMemory,
+                                                   offset,
+                                                   length,
+                                                   &pCpuVirtAddr,
+                                                   flags);
         if (NV_OK != rmStatus)
         {
             NV_PRINTF(LEVEL_WARNING,
-                      "Failed to flush GPU L2 (GPU 0x%llx): (rmStatus = %x)\n",
-                      pTargetGpu->busInfo.gpuPhysAddr, rmStatus);
-            goto cleanup_mapping;
+                      "Failed to map memory into internal smdbg client (GPU 0x%llx, hClient 0x%x, hMemory %x, offset 0x%llx, length 0x%x, flags 0x%x): (rmStatus = %x)\n",
+                      pTargetGpu->busInfo.gpuPhysAddr,
+                      hClient,
+                      hMemory,
+                      offset,
+                      length,
+                      flags,
+                      rmStatus);
+            return rmStatus;
         }
-    }
 
-    // Perform the requested accessType operation
-    if (accessType == GRDBG_MEM_ACCESS_TYPE_READ)
-    {
-        if (!portMemCopy(NvP64_VALUE(buffer), length, NvP64_VALUE(pCpuVirtAddr), length))
+        // Perform the requested accessType operation
+        if (accessType == GRDBG_MEM_ACCESS_TYPE_READ)
         {
-            rmStatus = NV_ERR_INVALID_ARGUMENT;
-            NV_PRINTF(LEVEL_WARNING,
-                      "portMemCopy failed (from VA 0x" NvP64_fmt " to 0x" NvP64_fmt ", length 0x%x)\n",
-                      pCpuVirtAddr, buffer, length);
-            goto cleanup_mapping;
+            if (!portMemCopy(NvP64_VALUE(buffer), length, NvP64_VALUE(pCpuVirtAddr), length))
+            {
+                rmStatus = NV_ERR_INVALID_ARGUMENT;
+                NV_PRINTF(LEVEL_WARNING,
+                          "portMemCopy failed (from VA 0x" NvP64_fmt " to 0x" NvP64_fmt ", length 0x%x)\n",
+                          pCpuVirtAddr, buffer, length);
+                goto cleanup_mapping;
+            }
+            NV_PRINTF(LEVEL_INFO, "Reading %d bytes of memory from 0x%x\n",
+                      length, hMemory);
         }
-        NV_PRINTF(LEVEL_INFO, "Reading %d bytes of memory from 0x%x\n",
-                  length, hMemory);
+        else
+        {
+            if (!portMemCopy(NvP64_VALUE(pCpuVirtAddr), length, NvP64_VALUE(buffer), length))
+            {
+                rmStatus = NV_ERR_INVALID_ARGUMENT;
+                NV_PRINTF(LEVEL_WARNING,
+                          "portMemCopy failed (from VA 0x" NvP64_fmt " to 0x" NvP64_fmt ", length 0x%x)\n",
+                          buffer, pCpuVirtAddr, length);
+                goto cleanup_mapping;
+            }
+
+            NV_PRINTF(LEVEL_INFO, "Writing %d bytes of memory to 0x%x\n", length,
+                      hMemory);
+        }
+
+cleanup_mapping:
+        // Unmap memory.
+        rmUnmapStatus = _nv83deUnmapMemoryFromGrdbgClient(pTargetGpu,
+                                                          pKernelSMDebuggerSession,
+                                                          pCpuVirtAddr,
+                                                          flags);
     }
     else
     {
-        if (!portMemCopy(NvP64_VALUE(pCpuVirtAddr), length, NvP64_VALUE(buffer), length))
-        {
-            rmStatus = NV_ERR_INVALID_ARGUMENT;
-            NV_PRINTF(LEVEL_WARNING,
-                      "portMemCopy failed (from VA 0x" NvP64_fmt " to 0x" NvP64_fmt ", length 0x%x)\n",
-                      buffer, pCpuVirtAddr, length);
-            goto cleanup_mapping;
-        }
+        MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pTargetGpu);
+        TRANSFER_SURFACE surf = { .pMemDesc = pMemDesc, .offset = offset };
+        // Prefer CE, but use BAR1 if not available; disable for maxwell due to undebugged issues in specific tests
+        NvU32 transferFlags = (IsMAXWELL(pTargetGpu) ? 0 : TRANSFER_FLAGS_PREFER_CE) |
+                              TRANSFER_FLAGS_USE_BAR1;
 
-        NV_PRINTF(LEVEL_INFO, "Writing %d bytes of memory to 0x%x\n", length,
-                  hMemory);
+        if (accessType == GRDBG_MEM_ACCESS_TYPE_READ)
+        {
+            NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                memmgrMemRead(pMemoryManager, &surf, NvP64_VALUE(buffer), length, transferFlags));
+        }
+        else
+        {
+            NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+                memmgrMemWrite(pMemoryManager, &surf, NvP64_VALUE(buffer), length, transferFlags));
+        }
     }
 
-    // Another fence to ensure our own new accesses are complete
-    osFlushCpuWriteCombineBuffer();
-
-cleanup_mapping:
-    // Unmap memory.
-    rmUnmapStatus = _nv83deUnmapMemoryFromGrdbgClient(pTargetGpu,
-                                                      pKernelSMDebuggerSession,
-                                                      pCpuVirtAddr,
-                                                      flags);
     // Return the first failure
     return (rmStatus != NV_OK ? rmStatus: rmUnmapStatus);
 }
@@ -681,27 +733,29 @@ NV_STATUS ksmdbgssnCtrlCmdDebugExecRegOps_IMPL
 )
 {
     OBJGPU *pGpu = GPU_RES_GET_GPU(pKernelSMDebuggerSession);
-    NV_STATUS status = NV_OK;
     NvBool isClientGspPlugin = NV_FALSE;
 
+    NV_CHECK_OR_RETURN(LEVEL_WARNING,
+        pParams->regOpCount <= NV83DE_CTRL_GPU_EXEC_REG_OPS_MAX_OPS,
+        NV_ERR_INVALID_ARGUMENT);
+
     // Check if User have permission to access register offset
     NV_CHECK_OK_OR_RETURN(LEVEL_INFO,
         gpuValidateRegOps(pGpu, pParams->regOps, pParams->regOpCount,
-                          pParams->bNonTransactional, isClientGspPlugin));
+                          pParams->bNonTransactional, isClientGspPlugin, NV_FALSE));
 
     if (IS_GSP_CLIENT(pGpu))
     {
-        CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
+        CALL_CONTEXT *pCallContext  = resservGetTlsCallContext();
         RmCtrlParams *pRmCtrlParams = pCallContext->pControlParams;
+        RM_API       *pRmApi        = GPU_GET_PHYSICAL_RMAPI(pGpu);
 
-        NV_RM_RPC_CONTROL(pGpu,
-                          pRmCtrlParams->hClient,
-                          pRmCtrlParams->hObject,
-                          pRmCtrlParams->cmd,
-                          pRmCtrlParams->pParams,
-                          pRmCtrlParams->paramsSize,
-                          status);
-        return status;
+        return pRmApi->Control(pRmApi,
+                               pRmCtrlParams->hClient,
+                               pRmCtrlParams->hObject,
+                               pRmCtrlParams->cmd,
+                               pRmCtrlParams->pParams,
+                               pRmCtrlParams->paramsSize);
     }
 
     return NV_ERR_NOT_SUPPORTED;
@@ -718,16 +772,18 @@ ksmdbgssnCtrlCmdDebugReadBatchMemory_IMPL
     NV_STATUS status = NV_OK;
     NvU32 i;
 
-    NV_CHECK_OR_RETURN(LEVEL_ERROR, pParams->count <= MAX_ACCESS_MEMORY_OPS,
+    NV_CHECK_OR_RETURN(LEVEL_WARNING, pParams->count <= MAX_ACCESS_MEMORY_OPS,
                        NV_ERR_INVALID_ARGUMENT);
 
     for (i = 0; i < pParams->count; ++i)
     {
         NV_STATUS localStatus = NV_OK;
         NvP64 pData = (NvP64)(((NvU8 *)pParams->pData) + pParams->entries[i].dataOffset);
+        NvU32 endingOffset;
 
-        NV_CHECK_OR_ELSE(LEVEL_ERROR,
-            pParams->entries[i].dataOffset < pParams->dataLength,
+        NV_CHECK_OR_ELSE(LEVEL_WARNING,
+            portSafeAddU32(pParams->entries[i].dataOffset, pParams->entries[i].length, &endingOffset) &&
+            (endingOffset <= pParams->dataLength),
             localStatus = NV_ERR_INVALID_OFFSET;
             goto updateStatus; );
 
@@ -762,13 +818,18 @@ ksmdbgssnCtrlCmdDebugWriteBatchMemory_IMPL
     NV_STATUS status = NV_OK;
     NvU32 i;
 
+    NV_CHECK_OR_RETURN(LEVEL_WARNING, pParams->count <= MAX_ACCESS_MEMORY_OPS,
+                       NV_ERR_INVALID_ARGUMENT);
+
     for (i = 0; i < pParams->count; ++i)
     {
         NV_STATUS localStatus = NV_OK;
         NvP64 pData = (NvP64)(((NvU8 *)pParams->pData) + pParams->entries[i].dataOffset);
+        NvU32 endingOffset;
 
-        NV_CHECK_OR_ELSE(LEVEL_ERROR,
-            (pParams->entries[i].dataOffset + pParams->entries[i].length) <= pParams->dataLength,
+        NV_CHECK_OR_ELSE(LEVEL_WARNING,
+            portSafeAddU32(pParams->entries[i].dataOffset, pParams->entries[i].length, &endingOffset) &&
+            (endingOffset <= pParams->dataLength),
             localStatus = NV_ERR_INVALID_OFFSET;
             goto updateStatus; );
 
@@ -802,7 +863,7 @@ ksmdbgssnCtrlCmdDebugReadAllSmErrorStates_IMPL
     NV_STATUS rmStatus = NV_OK;
     OBJGPU *pGpu = GPU_RES_GET_GPU(pKernelSMDebuggerSession);
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
     if (IS_VIRTUAL(pGpu))
     {
@@ -828,7 +889,7 @@ ksmdbgssnCtrlCmdDebugReadAllSmErrorStates_IMPL
         {
             NV_ASSERT_OK(
                 kgrctxLookupMmuFault(pGpu,
-                                     pKernelSMDebuggerSession->pObject->pKernelGraphicsContext,
+                                     kgrobjGetKernelGraphicsContext(pGpu, pKernelSMDebuggerSession->pObject),
                                      &pParams->mmuFault));
         }
 
@@ -848,7 +909,7 @@ ksmdbgssnCtrlCmdDebugClearAllSmErrorStates_IMPL
     NV_STATUS rmStatus = NV_OK;
     OBJGPU *pGpu = GPU_RES_GET_GPU(pKernelSMDebuggerSession);
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
 
     if (IS_VIRTUAL(pGpu))
     {
@@ -872,7 +933,7 @@ ksmdbgssnCtrlCmdDebugClearAllSmErrorStates_IMPL
         if (IS_VIRTUAL_WITH_SRIOV(pGpu))
         {
             NV_ASSERT_OK(
-                kgrctxClearMmuFault(pGpu, pKernelSMDebuggerSession->pObject->pKernelGraphicsContext));
+                kgrctxClearMmuFault(pGpu, kgrobjGetKernelGraphicsContext(pGpu, pKernelSMDebuggerSession->pObject)));
         }
 
         return rmStatus;
@@ -881,3 +942,36 @@ ksmdbgssnCtrlCmdDebugClearAllSmErrorStates_IMPL
     return NV_ERR_NOT_SUPPORTED;
 }
 
+NV_STATUS
+ksmdbgssnCtrlCmdDebugReadMMUFaultInfo_IMPL
+(
+    KernelSMDebuggerSession *pKernelSMDebuggerSession,
+    NV83DE_CTRL_DEBUG_READ_MMU_FAULT_INFO_PARAMS *pParams
+)
+{
+    OBJGPU *pGpu = GPU_RES_GET_GPU(pKernelSMDebuggerSession);
+
+    NV_ASSERT_OR_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner(), NV_ERR_INVALID_LOCK_STATE);
+
+    if (IS_GSP_CLIENT(pGpu))
+    {
+        RM_API *pRmApi = GPU_GET_PHYSICAL_RMAPI(pGpu);
+        CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
+        RmCtrlParams *pRmCtrlParams = pCallContext->pControlParams->pLegacyParams;
+
+        return pRmApi->Control(pRmApi,
+                                pRmCtrlParams->hClient,
+                                pRmCtrlParams->hObject,
+                                pRmCtrlParams->cmd,
+                                pRmCtrlParams->pParams,
+                                pRmCtrlParams->paramsSize);
+    }
+
+    NV_ASSERT_OK_OR_RETURN(
+        kgrctxLookupMmuFaultInfo(pGpu,
+                                 kgrobjGetKernelGraphicsContext(pGpu, pKernelSMDebuggerSession->pObject),
+                                 pParams));
+
+    return NV_OK;
+}
+
