@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2009-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2009-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -23,12 +23,15 @@
 
 #include "core/core.h"
 #include "os/os.h"
+#include "gpu_mgr/gpu_mgr.h"
 #include "gpu/mem_mgr/mem_mgr.h"
 #include "gpu/mem_mgr/fbsr.h"
 #include "gpu/bus/kern_bus.h"
 #include "gpu/mem_mgr/mem_desc.h"
+#include "gpu/gsp/kernel_gsp.h"
 #include "published/maxwell/gm107/dev_ram.h"
 #include "core/thread_state.h"
+#include "nvrm_registry.h"
 
 //
 // Implementation notes:
@@ -104,19 +107,22 @@ static NV_STATUS _fbsrInitGsp
     OBJFBSR *pFbsr
 )
 {
-    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
-    NvHandle       hSysMem        = NV01_NULL_OBJECT;
-    RM_API        *pRmApi         = GPU_GET_PHYSICAL_RMAPI(pGpu);
+    MemoryManager     *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+    KernelGsp         *pKernelGsp     = GPU_GET_KERNEL_GSP(pGpu);
+    NvHandle           hSysMem        = NV01_NULL_OBJECT;
+    RM_API            *pRmApi         = GPU_GET_PHYSICAL_RMAPI(pGpu);
+    MEMORY_DESCRIPTOR *pGspSysMemDesc = pMemoryManager->fbsrReservedRanges[FBSR_RESERVED_INST_MEMORY_GSP_HEAP];
     NV2080_CTRL_INTERNAL_FBSR_INIT_PARAMS params;
 
-    // Register sysmem memdesc with GSP. This creates memlist object
-    NV_ASSERT_OK_OR_RETURN(memdescSendMemDescToGSP(pGpu, pFbsr->pSysMemDesc, &hSysMem));
+    NV_ASSERT_OR_RETURN(pGspSysMemDesc != NULL, NV_ERR_INVALID_STATE);
 
-    params.fbsrType   = pFbsr->type;
-    params.numRegions = pFbsr->numRegions;
-    params.hClient    = pMemoryManager->hClient; 
+    // Register sysmem memdesc with GSP. This creates memlist object
+    NV_ASSERT_OK_OR_RETURN(memdescSendMemDescToGSP(pGpu, pGspSysMemDesc, &hSysMem));
+
+    params.hClient    = pMemoryManager->hClient;
     params.hSysMem    = hSysMem;
-    params.gspFbAllocsSysOffset = pFbsr->gspFbAllocsSysOffset;
+    params.bEnteringGcoffState  = pGpu->getProperty(pGpu, PDB_PROP_GPU_GCOFF_STATE_ENTERING);
+    params.sysmemAddrOfSuspendResumeData = memdescGetPhysAddr(pKernelGsp->pSRMetaDescriptor, AT_GPU, 0);
 
     // Send S/R init information to GSP
     NV_ASSERT_OK_OR_RETURN(pRmApi->Control(pRmApi,
@@ -129,54 +135,6 @@ static NV_STATUS _fbsrInitGsp
     // Free memlist object
     pRmApi->Free(pRmApi, pMemoryManager->hClient, hSysMem);
 
-    //
-    // Clear numRegions for next S/R sequence
-    // Needed only to tell GSP how many regions
-    //
-    pFbsr->numRegions = 0;
-
-    return NV_OK;
-}
-
-static NV_STATUS _fbsrMemoryCopy
-(
-    OBJGPU            *pGpu,
-    OBJFBSR           *pFbsr,
-    MEMORY_DESCRIPTOR *pDstMemDesc,
-    NvU64              dstOffset,
-    MEMORY_DESCRIPTOR *pSrcMemDesc,
-    NvU64              srcOffset,
-    NvU64              size
-)
-{
-    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
-    NvHandle       hVidMem        = NV01_NULL_OBJECT;
-    RM_API        *pRmApi         = GPU_GET_PHYSICAL_RMAPI(pGpu);
-    NV2080_CTRL_INTERNAL_FBSR_SEND_REGION_INFO_PARAMS params;
-
-    // Register vidmem memdesc with GSP. This creates memlist object
-    NV_ASSERT_OK_OR_RETURN(memdescSendMemDescToGSP(pGpu, pSrcMemDesc, &hVidMem));
-
-    portMemSet(&params, 0, sizeof(params));
-
-    params.fbsrType  = pFbsr->type;
-    params.hClient   = pMemoryManager->hClient; 
-    params.hVidMem   = hVidMem;
-    params.vidOffset = srcOffset;
-    params.sysOffset = dstOffset;
-    params.size      = size;
-
-    // Send region information to GSP
-    NV_ASSERT_OK_OR_RETURN(pRmApi->Control(pRmApi,
-                                           pGpu->hInternalClient,
-                                           pGpu->hInternalSubdevice,
-                                           NV2080_CTRL_CMD_INTERNAL_FBSR_SEND_REGION_INFO,
-                                           &params,
-                                           sizeof(params)));
-
-    // Free memlist object
-    pRmApi->Free(pRmApi, pMemoryManager->hClient, hVidMem);
-
     return NV_OK;
 }
 
@@ -252,7 +210,8 @@ fbsrInit_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr)
             goto fail;
         }
 
-        status = memdescAlloc(pFbsr->pSysMemDesc);
+        memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_126, 
+                        pFbsr->pSysMemDesc);
         if (status != NV_OK)
         {
             NV_ASSERT(status == NV_OK);
@@ -339,14 +298,17 @@ NV_STATUS
 fbsrBegin_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, FBSR_OP_TYPE op)
 {
     NV_STATUS status = NV_OK;
+    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
 
     pFbsr->op = op;
     pFbsr->bOperationFailed = NV_FALSE;
+
     if (op != FBSR_OP_SIZE_BUF && op != FBSR_OP_DESTROY)
     {
-        if (IS_GSP_CLIENT(pGpu))
+        if (IS_GSP_CLIENT(pGpu) || IS_VIRTUAL(pGpu))
         {
             pFbsr->pCe = NULL;
+            NV_ASSERT_OK_OR_RETURN(memmgrInitCeUtils(pMemoryManager, NV_FALSE));
         }
 
         NV_PRINTF(LEVEL_INFO, "%s %lld bytes of data\n",
@@ -416,7 +378,8 @@ fbsrBegin_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, FBSR_OP_TYPE op)
                         break;
                     }
 
-                    status = memdescAlloc(pFbsr->pSysMemDesc);
+                    memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_127, 
+                                    pFbsr->pSysMemDesc);
                     if (status != NV_OK)
                     {
                         NV_ASSERT(status == NV_OK);
@@ -451,9 +414,9 @@ fbsrBegin_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, FBSR_OP_TYPE op)
                         break;
                     }
 
-                    if(bIommuEnabled)
+                    if (bIommuEnabled)
                     {
-                        status = osSrPinSysmem(pGpu->pOsGpuInfo, 
+                        status = osSrPinSysmem(pGpu->pOsGpuInfo,
                                                     pFbsr->length,
                                                     &pFbsr->pagedBufferInfo.pMdl);
 
@@ -542,7 +505,7 @@ fbsrBegin_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, FBSR_OP_TYPE op)
         }
 
         // Initialize FBSR on GSP
-        if (IS_GSP_CLIENT(pGpu) && (pFbsr->pSysMemDesc != NULL))
+        if ((status == NV_OK) && IS_GSP_CLIENT(pGpu) && (pFbsr->pSysMemDesc != NULL))
         {
             NV_ASSERT_OK_OR_RETURN(_fbsrInitGsp(pGpu, pFbsr));
         }
@@ -656,9 +619,15 @@ fbsrEnd_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr)
 {
     NvBool bIommuEnabled = pGpu->getProperty(pGpu, PDB_PROP_GPU_ENABLE_IOMMU_SUPPORT);
     NV_STATUS status = NV_OK;
+    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
 
     if (pFbsr->op != FBSR_OP_SIZE_BUF && pFbsr->op != FBSR_OP_DESTROY)
     {
+
+        if ((IS_VIRTUAL(pGpu) || IS_GSP_CLIENT(pGpu)) && (pMemoryManager->pCeUtils != NULL))
+        {
+            memmgrDestroyCeUtils(pMemoryManager);
+        }
     }
 
     if (pFbsr->op == FBSR_OP_RESTORE || pFbsr->bOperationFailed || pFbsr->op == FBSR_OP_DESTROY)
@@ -795,6 +764,7 @@ fbsrEnd_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr)
 void
 fbsrCopyMemoryMemDesc_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, MEMORY_DESCRIPTOR *pVidMemDesc)
 {
+    MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
     NV_STATUS  status = NV_OK;
 
     NV_ASSERT(!gpumgrGetBcEnabledStatus(pGpu));
@@ -819,8 +789,6 @@ fbsrCopyMemoryMemDesc_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, MEMORY_DESCRIPTOR *pVi
 
     if (pFbsr->op == FBSR_OP_SIZE_BUF)
     {
-        pFbsr->numRegions++;
-
         switch (pFbsr->type)
         {
             case FBSR_TYPE_CPU:
@@ -866,7 +834,8 @@ fbsrCopyMemoryMemDesc_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, MEMORY_DESCRIPTOR *pVi
                                                MEMDESC_FLAGS_NONE);
                         if (status == NV_OK)
                         {
-                            status = memdescAlloc(pStandbyBuffer);
+                            memdescTagAlloc(status, NV_FB_ALLOC_RM_INTERNAL_OWNER_UNNAMED_TAG_128, 
+                                            pStandbyBuffer);
                             if (status != NV_OK )
                             {
                                 memdescDestroy(pStandbyBuffer);
@@ -900,27 +869,42 @@ fbsrCopyMemoryMemDesc_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, MEMORY_DESCRIPTOR *pVi
             case FBSR_TYPE_PAGED_DMA:
             case FBSR_TYPE_DMA:
                 {
+                    TRANSFER_SURFACE   vidSurface = {0};
+                    TRANSFER_SURFACE   sysSurface = {0};
+
+                    vidSurface.pMemDesc = pVidMemDesc;
+                    sysSurface.pMemDesc = pFbsr->pSysMemDesc;
+                    sysSurface.offset   = pFbsr->sysOffset;
+
                     if (pFbsr->op == FBSR_OP_RESTORE)
                     {
+                        NV_ASSERT_OK(memmgrMemCopy(pMemoryManager, &vidSurface, &sysSurface, pVidMemDesc->Size,
+                                                   TRANSFER_FLAGS_PREFER_CE | TRANSFER_FLAGS_CE_PRI_DEFER_FLUSH));
                     }
                     else
                     {
-                        if (IS_GSP_CLIENT(pGpu))
-                        {
-                            _fbsrMemoryCopy(pGpu, pFbsr, pFbsr->pSysMemDesc,
-                                pFbsr->sysOffset, pVidMemDesc, 0,
-                                pVidMemDesc->Size);
-                        }
+                        NV_ASSERT_OK(memmgrMemCopy(pMemoryManager, &sysSurface, &vidSurface, pVidMemDesc->Size,
+                                                   TRANSFER_FLAGS_PREFER_CE | TRANSFER_FLAGS_CE_PRI_DEFER_FLUSH));
                     }
                     break;
                 }
             case FBSR_TYPE_PERSISTENT:
                 {
+                    TRANSFER_SURFACE   vidSurface = {0};
+                    TRANSFER_SURFACE   sysSurface = {0};
+
+                    vidSurface.pMemDesc = pVidMemDesc;
+                    sysSurface.pMemDesc = memdescGetStandbyBuffer(pVidMemDesc);
+
                     if (pFbsr->op == FBSR_OP_RESTORE)
                     {
+                        NV_ASSERT_OK(memmgrMemCopy(pMemoryManager, &vidSurface, &sysSurface, pVidMemDesc->Size,
+                                                   TRANSFER_FLAGS_PREFER_CE | TRANSFER_FLAGS_CE_PRI_DEFER_FLUSH));
                     }
                     else
                     {
+                        NV_ASSERT_OK(memmgrMemCopy(pMemoryManager, &sysSurface, &vidSurface, pVidMemDesc->Size,
+                                                   TRANSFER_FLAGS_PREFER_CE | TRANSFER_FLAGS_CE_PRI_DEFER_FLUSH));
                     }
                     break;
                 }
@@ -975,11 +959,20 @@ fbsrCopyMemoryMemDesc_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, MEMORY_DESCRIPTOR *pVi
                             }
                         }
 
+                        TRANSFER_SURFACE vidSurface = {0};
+                        vidSurface.pMemDesc = pVidMemDesc;
+                        vidSurface.offset   = vidOffset;
+
+                        TRANSFER_SURFACE sysSurface = {0};
+                        sysSurface.pMemDesc = pFbsr->pSysMemDesc;
+
                         if (pFbsr->op == FBSR_OP_RESTORE)
                         {
+                            NV_ASSERT_OK(memmgrMemCopy(pMemoryManager, &vidSurface, &sysSurface, copySize, TRANSFER_FLAGS_PREFER_CE));
                         }
                         else
                         {
+                            NV_ASSERT_OK(memmgrMemCopy(pMemoryManager, &sysSurface, &vidSurface, copySize, TRANSFER_FLAGS_PREFER_CE));
                         }
 
                         if (pFbsr->op == FBSR_OP_SAVE)
@@ -1070,11 +1063,20 @@ fbsrCopyMemoryMemDesc_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, MEMORY_DESCRIPTOR *pVi
                             }
                         }
 
+                        TRANSFER_SURFACE vidSurface = {0};
+                        vidSurface.pMemDesc = pVidMemDesc;
+                        vidSurface.offset   = vidOffset;
+
+                        TRANSFER_SURFACE sysSurface = {0};
+                        sysSurface.pMemDesc = pFbsr->pSysMemDesc;
+
                         if (pFbsr->op == FBSR_OP_RESTORE)
                         {
+                            NV_ASSERT_OK(memmgrMemCopy(pMemoryManager, &vidSurface, &sysSurface, copySize, TRANSFER_FLAGS_PREFER_CE));
                         }
                         else
                         {
+                            NV_ASSERT_OK(memmgrMemCopy(pMemoryManager, &sysSurface, &vidSurface, copySize, TRANSFER_FLAGS_PREFER_CE));
                         }
 
                         if (pFbsr->op == FBSR_OP_SAVE)
@@ -1122,11 +1124,20 @@ fbsrCopyMemoryMemDesc_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, MEMORY_DESCRIPTOR *pVi
                             }
                         }
 
+                        TRANSFER_SURFACE vidSurface = {0};
+                        vidSurface.pMemDesc = pVidMemDesc;
+                        vidSurface.offset   = vidOffset;
+
+                        TRANSFER_SURFACE sysSurface = {0};
+                        sysSurface.pMemDesc = pFbsr->pSysMemDesc;
+
                         if (pFbsr->op == FBSR_OP_RESTORE)
                         {
+                            NV_ASSERT_OK(memmgrMemCopy(pMemoryManager, &vidSurface, &sysSurface, copySize, TRANSFER_FLAGS_PREFER_CE));
                         }
                         else
                         {
+                            NV_ASSERT_OK(memmgrMemCopy(pMemoryManager, &sysSurface, &vidSurface, copySize, TRANSFER_FLAGS_PREFER_CE));
                         }
 
                         if (pFbsr->op == FBSR_OP_SAVE)
@@ -1153,4 +1164,3 @@ fbsrCopyMemoryMemDesc_GM107(OBJGPU *pGpu, OBJFBSR *pFbsr, MEMORY_DESCRIPTOR *pVi
 
 #ifdef DEBUG
 #endif
-
