@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 1993-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 1993-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -21,6 +21,8 @@
  * DEALINGS IN THE SOFTWARE.
  */
 
+#define NVOC_KERNEL_NVLINK_H_PRIVATE_ACCESS_ALLOWED
+
 #include "gpu/gpu.h"
 #include "gpu/subdevice/subdevice.h"
 #include "vgpu/rpc.h"
@@ -60,13 +62,33 @@ subdeviceCtrlCmdBusGetNvlinkCaps_IMPL
     return nvlinkCtrlCmdBusGetNvlinkCaps(pGpu, pParams);
 }
 
+NV_STATUS
+subdeviceCtrlCmdBusGetNvlinkCaps_VF
+(
+    Subdevice *pSubdevice,
+    NV2080_CTRL_CMD_NVLINK_GET_NVLINK_CAPS_PARAMS *pParams
+)
+{
+    OBJGPU *pGpu = GPU_RES_GET_GPU(pSubdevice);
+    VGPU_STATIC_INFO *pVSI = GPU_GET_STATIC_INFO(pGpu);
+
+    NV_ASSERT_OR_RETURN(pVSI != NULL, NV_ERR_INVALID_STATE);
+
+    portMemCopy(pParams, sizeof(*pParams),
+                &pVSI->nvlinkCaps, sizeof(pVSI->nvlinkCaps));
+
+    return NV_OK;
+}
+
 static void _calculateNvlinkCaps
 (
     OBJGPU *pGpu,
-    NvU32   bridgeSensableLinks,
+    NvU64   bridgeSensableLinks,
     NvU32   bridgedLinks,
     NvU32   ipVerNvlink,
     NvBool  bMIGNvLinkP2PSupported,
+    NvBool  bNvlinkEnabled,
+    NvBool  bIsUncontainedErrorRecoveryCapable,
     NV2080_CTRL_CMD_NVLINK_GET_NVLINK_CAPS_PARAMS *pParams
 )
 {
@@ -101,10 +123,28 @@ static void _calculateNvlinkCaps
         {
             RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _SYSMEM_ATOMICS);
         }
+
+        if (bNvlinkEnabled && bIsUncontainedErrorRecoveryCapable)
+        {
+            RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _UNCONTAINED_ERROR_RECOVERY);
+        }
     }
 
     switch (ipVerNvlink)
     {
+        case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_5_0:
+        {
+            pParams->lowestNvlinkVersion  = NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_5_0;
+            pParams->highestNvlinkVersion = NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_5_0;
+            pParams->lowestNciVersion     = NV2080_CTRL_NVLINK_CAPS_NCI_VERSION_5_0;
+            pParams->highestNciVersion    = NV2080_CTRL_NVLINK_CAPS_NCI_VERSION_5_0;
+
+            // Supported power states
+            RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _POWER_STATE_L0);
+            RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _POWER_STATE_L1);
+            RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _POWER_STATE_L2);
+            break;
+        }
         case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_4_0:
         {
             pParams->lowestNvlinkVersion  = NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_4_0;
@@ -177,6 +217,11 @@ static void _calculateNvlinkCaps
         }
     }
 
+    if (bNvlinkEnabled)
+    {
+        RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _VALID);
+    }
+
     portMemCopy(&pParams->capsTbl, NV2080_CTRL_NVLINK_CAPS_TBL_SIZE, tempCaps, NV2080_CTRL_NVLINK_CAPS_TBL_SIZE);
 }
 
@@ -195,23 +240,6 @@ nvlinkCtrlCmdBusGetNvlinkCaps
     KernelMIGManager *pKernelMIGManager      = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
     NvBool            bMIGNvLinkP2PSupported = ((pKernelMIGManager != NULL) &&
                                                 kmigmgrIsMIGNvlinkP2PSupported(pGpu, pKernelMIGManager));
-    //
-    // vGPU:
-    //
-    // Since vGPU does all real hardware management in the
-    // host, if we are in guest OS (where IS_VIRTUAL(pGpu) is true),
-    // do an RPC to the host to get blacklist information from host RM
-    //
-    if (IS_VIRTUAL(pGpu))
-    {
-        CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
-        RmCtrlParams *pRmCtrlParams = pCallContext->pControlParams;
-        NV_STATUS     status = NV_OK;
-
-        NV_RM_RPC_CONTROL(pGpu, pRmCtrlParams->hClient, pRmCtrlParams->hObject, pRmCtrlParams->cmd,
-                          pRmCtrlParams->pParams, pRmCtrlParams->paramsSize, status);
-        return status;
-    }
 
     // Initialize link masks to 0
     pParams->enabledLinkMask    = 0;
@@ -239,10 +267,16 @@ nvlinkCtrlCmdBusGetNvlinkCaps
             //
             knvlinkFilterBridgeLinks_HAL(pGpu, pKernelNvlink);
         }
-        _calculateNvlinkCaps(pGpu, pKernelNvlink->bridgeSensableLinks, pKernelNvlink->bridgedLinks, pKernelNvlink->ipVerNvlink, bMIGNvLinkP2PSupported, pParams);
+        _calculateNvlinkCaps(pGpu, pKernelNvlink->bridgeSensableLinks, pKernelNvlink->bridgedLinks,
+                             pKernelNvlink->ipVerNvlink, bMIGNvLinkP2PSupported,
+                             pKernelNvlink->getProperty(pNvlink, PDB_PROP_KNVLINK_ENABLED),
+                             pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_UNCONTAINED_ERROR_RECOVERY_SUPPORTED),
+                             pParams);
 
-        pParams->discoveredLinkMask = knvlinkGetDiscoveredLinkMask(pGpu, pKernelNvlink);
-        pParams->enabledLinkMask    = knvlinkGetEnabledLinkMask(pGpu, pKernelNvlink);
+        pParams->discoveredLinkMask = KNVLINK_GET_MASK(pKernelNvlink, discoveredLinks, 32);
+        pParams->discoveredLinks    = knvlinkGetDiscoveredLinkMask(pGpu, pKernelNvlink);
+        pParams->enabledLinkMask    = KNVLINK_GET_MASK(pKernelNvlink, enabledLinks, 32);
+        pParams->enabledLinks       = knvlinkGetEnabledLinkMask(pGpu, pKernelNvlink);
     }
 
     return NV_OK;
@@ -252,14 +286,17 @@ void
 static _getNvlinkStatus
 (
     OBJGPU *pGpu,
-    NV2080_CTRL_NVLINK_GET_LINK_AND_CLOCK_INFO_PARAMS *nvlinkLinkAndClockInfoParams,
-    NvU32 bridgeSensableLinks,
+    NV2080_CTRL_INTERNAL_NVLINK_GET_LINK_AND_CLOCK_INFO_PARAMS *nvlinkLinkAndClockInfoParams,
+    NvU64 bridgeSensableLinks,
     NvU32 bridgedLinks,
     NvU32 ipVerNvlink,
     NvlinkLinkStatus nvlinkLinks[NVLINK_MAX_LINKS_SW],
     NvBool bNvlinkEnabled,
     NvBool bL2PowerStateEnabled,
     NvBool bForcedConfig,
+    NvBool bIsUncontainedErrorRecoveryCapable,
+    NvBool bIsGpuDegraded,
+    NvBool bIsUncontainedErrorRecoveryActive,
     NV2080_CTRL_CMD_NVLINK_GET_NVLINK_STATUS_PARAMS *pParams
 )
 {
@@ -279,7 +316,7 @@ static _getNvlinkStatus
         bPeerLink   = NV_FALSE;
         bSysmemLink = NV_FALSE;
         bSwitchLink = NV_FALSE;
-        NV2080_CTRL_NVLINK_GET_LINK_AND_CLOCK_VALUES *pLinkAndClockValues;
+        NV2080_CTRL_INTERNAL_NVLINK_GET_LINK_AND_CLOCK_VALUES *pLinkAndClockValues;
 
         pLinkAndClockValues = &nvlinkLinkAndClockInfoParams->linkInfo[i];
 
@@ -325,13 +362,18 @@ static _getNvlinkStatus
             RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _SLI_BRIDGE);
         }
 
+        if (bNvlinkEnabled && bIsUncontainedErrorRecoveryCapable)
+        {
+            RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _UNCONTAINED_ERROR_RECOVERY);
+        }
+
         // Set the power states caps
         switch (ipVerNvlink)
         {
             case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_2_0:
                 RMCTRL_SET_CAP(tempCaps, NV2080_CTRL_NVLINK_CAPS, _POWER_STATE_L0);
                 break;
-
+            case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_5_0:
             case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_4_0:
             case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_3_1:
             case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_3_0:
@@ -361,6 +403,10 @@ static _getNvlinkStatus
 
         switch (ipVerNvlink)
         {
+            case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_5_0:
+                pParams->linkInfo[i].nvlinkVersion = NV2080_CTRL_NVLINK_STATUS_NVLINK_VERSION_5_0;
+                pParams->linkInfo[i].nciVersion    = NV2080_CTRL_NVLINK_STATUS_NCI_VERSION_5_0;
+                break;
             case NV2080_CTRL_NVLINK_CAPS_NVLINK_VERSION_4_0:
                 pParams->linkInfo[i].nvlinkVersion = NV2080_CTRL_NVLINK_STATUS_NVLINK_VERSION_4_0;
                 pParams->linkInfo[i].nciVersion    = NV2080_CTRL_NVLINK_STATUS_NCI_VERSION_4_0;
@@ -411,6 +457,20 @@ static _getNvlinkStatus
                         pParams->linkInfo[i].localDeviceInfo.deviceIdFlags);
         }
 
+        pParams->linkInfo[i].localDeviceInfo.fabricRecoveryStatusMask = 0x0;
+        if (bIsGpuDegraded)
+        {
+            pParams->linkInfo[i].localDeviceInfo.fabricRecoveryStatusMask =
+                FLD_SET_DRF(2080_CTRL_NVLINK_DEVICE_INFO, _FABRIC_RECOVERY_STATUS_MASK, _GPU_DEGRADED, _TRUE,
+                            pParams->linkInfo[i].localDeviceInfo.fabricRecoveryStatusMask);
+        }
+        else if (bIsUncontainedErrorRecoveryActive)
+        {
+            pParams->linkInfo[i].localDeviceInfo.fabricRecoveryStatusMask =
+                FLD_SET_DRF(2080_CTRL_NVLINK_DEVICE_INFO, _FABRIC_RECOVERY_STATUS_MASK, _UNCONTAINED_ERROR_RECOVERY, _ACTIVE,
+                            pParams->linkInfo[i].localDeviceInfo.fabricRecoveryStatusMask);
+        }
+
         //
         // Get clock related state
         // NOTE: to be depricated HW terminology conforming versions
@@ -428,6 +488,10 @@ static _getNvlinkStatus
         pParams->linkInfo[i].nvlinkRefClkType        = pLinkAndClockValues->nvlinkRefClkType;
         pParams->linkInfo[i].nvlinkRefClkSpeedMhz    = pLinkAndClockValues->nvlinkReqLinkClockMhz;
 
+        pParams->linkInfo[i].nvlinkMinL1Threshold    = pLinkAndClockValues->nvlinkMinL1Threshold;
+        pParams->linkInfo[i].nvlinkMaxL1Threshold    = pLinkAndClockValues->nvlinkMaxL1Threshold;
+        pParams->linkInfo[i].nvlinkL1ThresholdUnits  = pLinkAndClockValues->nvlinkL1ThresholdUnits;
+
         if (nvlinkLinks[i].bConnected)
         {
             pParams->linkInfo[i].connected              = NV2080_CTRL_NVLINK_STATUS_CONNECTED_TRUE;
@@ -593,7 +657,7 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
     NvU8 i = 0;
     struct
     {
-        NV2080_CTRL_NVLINK_GET_LINK_AND_CLOCK_INFO_PARAMS nvlinkLinkAndClockInfoParams;
+        NV2080_CTRL_INTERNAL_NVLINK_GET_LINK_AND_CLOCK_INFO_PARAMS nvlinkLinkAndClockInfoParams;
         NvlinkLinkStatus nvlinkLinks[NVLINK_MAX_LINKS_SW];
     } *pTmpData = NULL;
 
@@ -622,7 +686,9 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
                 FOR_EACH_INDEX_IN_MASK(32, i, pParams->enabledLinkMask)
                 {
                     NV2080_CTRL_NVLINK_DEVICE_INFO *pDeviceInfo = &pParams->linkInfo[i].remoteDeviceInfo;
-                    OBJGPU *pLoopGpu = gpumgrGetGpuFromUuid(pDeviceInfo->deviceUUID, 
+                    if (pDeviceInfo->deviceType == NV2080_CTRL_NVLINK_DEVICE_INFO_DEVICE_TYPE_SWITCH)
+                    	continue;
+                    OBJGPU *pLoopGpu = gpumgrGetGpuFromUuid(pDeviceInfo->deviceUUID,
                                                             DRF_DEF(2080_GPU_CMD, _GPU_GET_GID_FLAGS, _TYPE, _SHA1) |
                                                             DRF_DEF(2080_GPU_CMD, _GPU_GET_GID_FLAGS, _FORMAT, _BINARY));
 
@@ -657,7 +723,7 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
 
     if (!bMIGNvLinkP2PSupported)
     {
-        NV_PRINTF(LEVEL_ERROR, "MIG NVLink P2P is not supported.\n");
+        NV_PRINTF(LEVEL_INFO, "MIG NVLink P2P is not supported.\n");
         status = NV_OK;
         return status;
     }
@@ -717,13 +783,13 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
         knvlinkFilterBridgeLinks_HAL(pGpu, pKernelNvlink);
 
         // If nvlink is not ready don't report back any links as being enabled
-        pParams->enabledLinkMask = (bIsNvlinkReady) ? pKernelNvlink->enabledLinks : 0x0;
+        pParams->enabledLinkMask = (bIsNvlinkReady) ? KNVLINK_GET_MASK(pKernelNvlink, enabledLinks, 32) : 0x0;
 
         pTmpData->nvlinkLinkAndClockInfoParams.linkMask = pParams->enabledLinkMask;
-        pTmpData->nvlinkLinkAndClockInfoParams.bSublinkStateInst = pParams->bSublinkStateInst;
+        pTmpData->nvlinkLinkAndClockInfoParams.bSublinkStateInst = NV_TRUE;
 
         status = knvlinkExecGspRmRpc(pGpu, pKernelNvlink,
-                                     NV2080_CTRL_CMD_NVLINK_GET_LINK_AND_CLOCK_INFO,
+                                     NV2080_CTRL_CMD_INTERNAL_NVLINK_GET_LINK_AND_CLOCK_INFO,
                                      (void *)&pTmpData->nvlinkLinkAndClockInfoParams,
                                      sizeof(pTmpData->nvlinkLinkAndClockInfoParams));
         if (status != NV_OK)
@@ -765,6 +831,9 @@ subdeviceCtrlCmdBusGetNvlinkStatus_IMPL
                          pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_ENABLED),
                          pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_L2_POWER_STATE_ENABLED),
                          knvlinkIsForcedConfig(pGpu, pKernelNvlink),
+                         pKernelNvlink->getProperty(pKernelNvlink, PDB_PROP_KNVLINK_UNCONTAINED_ERROR_RECOVERY_SUPPORTED),
+                         knvlinkGetDegradedMode(pGpu, pKernelNvlink),
+                         knvlinkIsUncontainedErrorRecoveryActive(pGpu, pKernelNvlink),
                          pParams);
     }
 done:
