@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2016-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2016-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -65,12 +65,21 @@ ctxBufPoolIsSupported
         return NV_FALSE;
     }
 
-    if (!memmgrIsPmaInitialized(pMemoryManager))
+    if (!memmgrIsPmaEnabled(pMemoryManager) || !memmgrIsPmaSupportedOnPlatform(pMemoryManager))
     {
         NV_PRINTF(LEVEL_INFO, "PMA is disabled. Ctx buffers will be allocated in RM reserved heap\n");
         return NV_FALSE;
     }
 
+    // TODO remove when bug ID 3922001 for ap_sim_compute_uvm test case resolved
+    if (!IS_SILICON(pGpu) &&
+        memmgrBug3922001DisableCtxBufOnSim(pGpu, pMemoryManager) &&
+        gpuIsSelfHosted(pGpu))
+    {
+        NV_PRINTF(LEVEL_INFO, "Ctx buffers not supported on simulation/emulation\n");
+        return NV_FALSE;
+    }
+
     if (IS_VIRTUAL(pGpu) || RMCFG_FEATURE_PLATFORM_GSP)
     {
         NV_PRINTF(LEVEL_INFO, "Guest RM/GSP don't support ctx buffers in PMA\n");
@@ -112,7 +121,7 @@ ctxBufPoolInit
 {
     NV_STATUS status = NV_OK;
     CTX_BUF_POOL_INFO *pCtxBufPool = NULL;
-    NvU32 i, poolConfig;
+    NvU32 i;
 
     NV_ASSERT_OR_RETURN(ppCtxBufPool != NULL, NV_ERR_INVALID_ARGUMENT);
 
@@ -127,34 +136,20 @@ ctxBufPoolInit
 
     //
     // create a mem pool for each page size supported by RM
-    // pool corresponding to RM_ATTR_PAGE_SIZE_DEFAULT remains unused
     //
-    for (i = 0; i < RM_ATTR_PAGE_SIZE_INVALID; i++)
+    for (i = 0; i < POOL_CONFIG_MAX_SUPPORTED; i++)
     {
-        switch (i)
-        {
-            case RM_ATTR_PAGE_SIZE_DEFAULT:
-            case RM_ATTR_PAGE_SIZE_4KB:
-                poolConfig = POOL_CONFIG_CTXBUF_4K;
-                break;
-            case RM_ATTR_PAGE_SIZE_BIG:
-                poolConfig = POOL_CONFIG_CTXBUF_64K;
-                break;
-            case RM_ATTR_PAGE_SIZE_HUGE:
-                poolConfig = POOL_CONFIG_CTXBUF_2M;
-                break;
-            case RM_ATTR_PAGE_SIZE_512MB:
-                poolConfig = POOL_CONFIG_CTXBUF_512M;
-                break;
-            default:
-                NV_PRINTF(LEVEL_ERROR, "Unsupported page size attr %d\n", i);
-                return NV_ERR_INVALID_STATE;
-        }
+        // Pool Config starts from POOL_CONFIG_CTXBUF_256G
         NV_ASSERT_OK_OR_GOTO(status,
             rmMemPoolSetup((void*)&pHeap->pmaObject, &pCtxBufPool->pMemPool[i],
-                           poolConfig),
+                           (POOL_CONFIG_MODE) i),
             cleanup);
 
+        // Allocate the pool in CPR in case of Confidential Compute
+        if (gpuIsCCFeatureEnabled(pGpu))
+        {
+            rmMemPoolAllocateProtectedMemory(pCtxBufPool->pMemPool[i], NV_TRUE);
+        }
     }
     NV_PRINTF(LEVEL_INFO, "Ctx buf pool successfully initialized\n");
 
@@ -194,7 +189,7 @@ ctxBufPoolDestroy
 
     pCtxBufPool = *ppCtxBufPool;
 
-    for (i = 0; i < RM_ATTR_PAGE_SIZE_INVALID; i++)
+    for (i = 0; i < POOL_CONFIG_MAX_SUPPORTED; i++)
     {
         if (pCtxBufPool->pMemPool[i] != NULL)
         {
@@ -207,6 +202,29 @@ ctxBufPoolDestroy
     NV_PRINTF(LEVEL_INFO, "Ctx buf pool destroyed\n");
 }
 
+static NvU32 NV_FORCEINLINE
+ctxBufPoolPageSizeToPoolIndex(NvU64 pageSize)
+{
+    switch (pageSize)
+    {
+        case RM_PAGE_SIZE:
+            return POOL_CONFIG_CTXBUF_4K;
+        case RM_PAGE_SIZE_64K:
+            return POOL_CONFIG_CTXBUF_64K;
+        case RM_PAGE_SIZE_128K:
+            return POOL_CONFIG_CTXBUF_128K;
+        case RM_PAGE_SIZE_HUGE:
+            return POOL_CONFIG_CTXBUF_2M;
+        case RM_PAGE_SIZE_512M:
+            return POOL_CONFIG_CTXBUF_512M;
+        case RM_PAGE_SIZE_256G:
+            return POOL_CONFIG_CTXBUF_256G;
+        default:
+            NV_PRINTF(LEVEL_ERROR, "Unrecognized/unsupported page size = 0x%llx\n", pageSize);
+            NV_ASSERT_OR_RETURN(0, POOL_CONFIG_MAX_SUPPORTED);
+    }
+}
+
 /*
  * @brief Calculates total amount of memory required for all buffers in each pool and reserves the memory
  *
@@ -246,7 +264,7 @@ ctxBufPoolReserve
     NV_STATUS status = NV_OK;
     NvU64 pageSize;
     NvU32 i;
-    NvU64 totalSize[RM_ATTR_PAGE_SIZE_INVALID] = {0};
+    NvU64 totalSize[POOL_CONFIG_MAX_SUPPORTED] = {0};
     NvU64 size;
 
     NV_ASSERT_OR_RETURN(pCtxBufPool != NULL, NV_ERR_INVALID_ARGUMENT);
@@ -265,29 +283,13 @@ ctxBufPoolReserve
         // Determine the pool(4K/64K/2M) from where this buffer will eventually
         // get allocated and mark that pool to reserve this memory.
         //
-        switch(pageSize)
-        {
-            case RM_PAGE_SIZE:
-                totalSize[RM_ATTR_PAGE_SIZE_4KB] += size;
-                break;
-            case RM_PAGE_SIZE_64K:
-            case RM_PAGE_SIZE_128K:
-                totalSize[RM_ATTR_PAGE_SIZE_BIG] += size;
-                break;
-            case RM_PAGE_SIZE_HUGE:
-                totalSize[RM_ATTR_PAGE_SIZE_HUGE] += size;
-                break;
-            case RM_PAGE_SIZE_512M:
-                totalSize[RM_ATTR_PAGE_SIZE_512MB] += size;
-                break;
-            default:
-                NV_PRINTF(LEVEL_ERROR, "Unrecognized/unsupported page size = 0x%llx\n", pageSize);
-                NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_ARGUMENT);
-        }
+        NvU32 poolIndex = ctxBufPoolPageSizeToPoolIndex(pageSize);
+        NV_ASSERT_OR_RETURN(poolIndex < POOL_CONFIG_MAX_SUPPORTED, NV_ERR_INVALID_ARGUMENT);
+        totalSize[poolIndex] += size;
         NV_PRINTF(LEVEL_INFO, "Reserving 0x%llx bytes for buf Id = 0x%x in pool with page size = 0x%llx\n", size, i, pageSize);
     }
 
-    for (i = 0; i < RM_ATTR_PAGE_SIZE_INVALID; i++)
+    for (i = 0; i < POOL_CONFIG_MAX_SUPPORTED; i++)
     {
         if (totalSize[i] > 0)
         {
@@ -322,7 +324,7 @@ ctxBufPoolTrim
     NvU32 i;
     NV_ASSERT_OR_RETURN(pCtxBufPool != NULL, NV_ERR_INVALID_ARGUMENT);
 
-    for (i = 0; i < RM_ATTR_PAGE_SIZE_INVALID; i++)
+    for (i = 0; i < POOL_CONFIG_MAX_SUPPORTED; i++)
     {
         rmMemPoolTrim(pCtxBufPool->pMemPool[i], 0, 0);
         NV_PRINTF(LEVEL_INFO, "Trimmed pool with RM_ATTR_PAGE_SIZE_* = 0x%x\n", i);
@@ -349,7 +351,7 @@ ctxBufPoolRelease
     NvU32 i;
     NV_ASSERT(pCtxBufPool != NULL);
 
-    for (i = 0; i < RM_ATTR_PAGE_SIZE_INVALID; i++)
+    for (i = 0; i < POOL_CONFIG_MAX_SUPPORTED; i++)
     {
         rmMemPoolRelease(pCtxBufPool->pMemPool[i], 0);
     }
@@ -406,26 +408,10 @@ ctxBufPoolAllocate
         pageSize = newPageSize;
     }
 
-    // Determine the pool(4K/64K/2M) from where this buffer is to be allocated
-    switch(pageSize)
-    {
-        case RM_PAGE_SIZE:
-            pPool = pCtxBufPool->pMemPool[RM_ATTR_PAGE_SIZE_4KB];
-            break;
-        case RM_PAGE_SIZE_64K:
-        case RM_PAGE_SIZE_128K:
-            pPool = pCtxBufPool->pMemPool[RM_ATTR_PAGE_SIZE_BIG];
-            break;
-        case RM_PAGE_SIZE_HUGE:
-            pPool = pCtxBufPool->pMemPool[RM_ATTR_PAGE_SIZE_HUGE];
-            break;
-        case RM_PAGE_SIZE_512M:
-            pPool = pCtxBufPool->pMemPool[RM_ATTR_PAGE_SIZE_512MB];
-            break;
-        default:
-            NV_PRINTF(LEVEL_ERROR, "Unsupported page size = 0x%llx set for context buffer\n", pageSize);
-            NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_ARGUMENT);
-    }
+    NvU32 poolIndex = ctxBufPoolPageSizeToPoolIndex(pageSize);
+    NV_ASSERT_OR_RETURN(poolIndex < POOL_CONFIG_MAX_SUPPORTED, NV_ERR_INVALID_ARGUMENT);
+    pPool = pCtxBufPool->pMemPool[poolIndex];
+
     NV_ASSERT_OK_OR_RETURN(rmMemPoolAllocate(pPool, (RM_POOL_ALLOC_MEMDESC*)pMemDesc));
     NV_PRINTF(LEVEL_INFO, "Buffer allocated from ctx buf pool with page size = 0x%llx\n", pageSize);
     return NV_OK;
@@ -465,41 +451,18 @@ ctxBufPoolFree
             pMemDesc->Alignment, RM_ATTR_PAGE_SIZE_DEFAULT, NV_TRUE, &size, &pageSize));
     }
 
-    switch(pageSize)
-    {
-        case RM_PAGE_SIZE:
-            pPool = pCtxBufPool->pMemPool[RM_ATTR_PAGE_SIZE_4KB];
-            break;
-        case RM_PAGE_SIZE_64K:
-        case RM_PAGE_SIZE_128K:
-            pPool = pCtxBufPool->pMemPool[RM_ATTR_PAGE_SIZE_BIG];
-            break;
-        case RM_PAGE_SIZE_HUGE:
-            pPool = pCtxBufPool->pMemPool[RM_ATTR_PAGE_SIZE_HUGE];
-            break;
-        case RM_PAGE_SIZE_512M:
-            pPool = pCtxBufPool->pMemPool[RM_ATTR_PAGE_SIZE_512MB];
-            break;
-        default:
-            NV_PRINTF(LEVEL_ERROR, "Unsupported page size detected for context buffer\n");
-            NV_ASSERT_OR_RETURN(0, NV_ERR_INVALID_STATE);
-    }
+    NvU32 poolIndex = ctxBufPoolPageSizeToPoolIndex(pageSize);
+    NV_ASSERT_OR_RETURN(poolIndex < POOL_CONFIG_MAX_SUPPORTED, NV_ERR_INVALID_ARGUMENT);
+    pPool = pCtxBufPool->pMemPool[poolIndex];
 
     // If scrubber is being skipped by PMA we need to manually scrub this memory
     if (rmMemPoolIsScrubSkipped(pPool))
     {
         OBJGPU *pGpu = pMemDesc->pGpu;
-        NvU8   *pMem = kbusMapRmAperture_HAL(pGpu, pMemDesc);
-        if (pMem == NULL)
-        {
-            NV_PRINTF(LEVEL_ERROR, "Failed to BAR2 map memdesc. memory won't be scrubbed\n");
-            NV_ASSERT(pMem != NULL);
-        }
-        else
-        {
-            portMemSet(pMem, 0, (pMemDesc->PageCount * RM_PAGE_SIZE));
-            kbusUnmapRmAperture_HAL(pGpu, pMemDesc, &pMem, NV_TRUE);
-        }
+        MemoryManager *pMemoryManager = GPU_GET_MEMORY_MANAGER(pGpu);
+
+        memmgrMemsetInBlocks(pMemoryManager, pMemDesc, 0, 0, pMemDesc->Size,
+                                    TRANSFER_FLAGS_NONE, 0 /* Default Block Size */);
     }
     rmMemPoolFree(pPool, (RM_POOL_ALLOC_MEMDESC*)pMemDesc, 0);
 
@@ -609,6 +572,10 @@ ctxBufPoolGetSizeAndPageSize
             retAttr = FLD_SET_DRF(OS32, _ATTR, _PAGE_SIZE, _HUGE, retAttr);
             retAttr2 = FLD_SET_DRF(OS32, _ATTR2, _PAGE_SIZE_HUGE, _512MB, retAttr2);
             break;
+        case RM_ATTR_PAGE_SIZE_256GB:
+            retAttr = FLD_SET_DRF(OS32, _ATTR, _PAGE_SIZE, _HUGE, retAttr);
+            retAttr2 = FLD_SET_DRF(OS32, _ATTR2, _PAGE_SIZE_HUGE, _256GB, retAttr2);
+            break;
         default:
             NV_PRINTF(LEVEL_ERROR, "unsupported page size attr\n");
             return NV_ERR_NOT_SUPPORTED;
@@ -642,16 +609,19 @@ ctxBufPoolGetSizeAndPageSize
     {
         NvU64 chunkSize = 0;
         NvU32 i;
-        for (i = 0; i < RM_ATTR_PAGE_SIZE_INVALID; i++)
+        //
+        // pools are sorted in descending order of chunk size. So, start from the pool with the smallest chunk size.
+        //
+        for (i = POOL_CONFIG_MAX_SUPPORTED; i; i--)
         {
-            NV_ASSERT_OK_OR_RETURN(rmMemPoolGetChunkAndPageSize(pCtxBufPool->pMemPool[i], &chunkSize, &pageSize));
+            NV_ASSERT_OK_OR_RETURN(rmMemPoolGetChunkAndPageSize(pCtxBufPool->pMemPool[i - 1], &chunkSize, &pageSize));
             if (chunkSize >= size)
             {
                 size = chunkSize;
                 break;
             }
         }
-        if (i == RM_ATTR_PAGE_SIZE_INVALID)
+        if (i == 0)
         {
             NV_PRINTF(LEVEL_ERROR, "couldn't find pool with chunksize >= 0x%llx\n", size);
             DBG_BREAKPOINT();
@@ -699,7 +669,7 @@ ctxBufPoolIsScrubSkipped
 {
     NvU32 i;
     NV_ASSERT_OR_RETURN(pCtxBufPool != NULL, NV_ERR_INVALID_ARGUMENT);
-    for (i = 0; i < RM_ATTR_PAGE_SIZE_INVALID; i++)
+    for (i = 0; i < POOL_CONFIG_MAX_SUPPORTED; i++)
     {
         if (!rmMemPoolIsScrubSkipped(pCtxBufPool->pMemPool[i]))
             return NV_FALSE;
@@ -724,7 +694,7 @@ ctxBufPoolSetScrubSkip
 {
     NvU32 i;
     NV_ASSERT_OR_RETURN_VOID(pCtxBufPool != NULL);
-    for (i = 0; i < RM_ATTR_PAGE_SIZE_INVALID; i++)
+    for (i = 0; i < POOL_CONFIG_MAX_SUPPORTED; i++)
     {
         rmMemPoolSkipScrub(pCtxBufPool->pMemPool[i], bSkipScrub);
     }
